import os
import shutil
import unittest

from sherlockpipe.star.starinfo import StarInfo

from sherlockpipe.objectinfo.InputObjectInfo import InputObjectInfo
from sherlockpipe.objectinfo.MissionFfiCoordsObjectInfo import MissionFfiCoordsObjectInfo
from sherlockpipe.objectinfo.MissionFfiIdObjectInfo import MissionFfiIdObjectInfo
from sherlockpipe.objectinfo.MissionInputObjectInfo import MissionInputObjectInfo
from sherlockpipe.objectinfo.MissionObjectInfo import MissionObjectInfo
from sherlockpipe.objectinfo.preparer.MissionFfiLightcurveBuilder import MissionFfiLightcurveBuilder
from sherlockpipe.objectinfo.preparer.MissionInputLightcurveBuilder import MissionInputLightcurveBuilder
from sherlockpipe.objectinfo.preparer.MissionLightcurveBuilder import MissionLightcurveBuilder
from sherlockpipe.scoring.BasicSignalSelector import BasicSignalSelector
from sherlockpipe.scoring.QuorumSnrBorderCorrectedSignalSelector import QuorumSnrBorderCorrectedSignalSelector
from sherlockpipe.scoring.SnrBorderCorrectedSignalSelector import SnrBorderCorrectedSignalSelector
from sherlockpipe.sherlock import Sherlock


class SherlockTests(unittest.TestCase):
    def test_setup_files(self):
        sherlock = Sherlock(False, None)
        sherlock.setup_files(False, "inner/")
        self.assertEqual("inner/", sherlock.results_dir)

    def test_setup_detrend(self):
        sherlock = Sherlock(False, None)
        sherlock.setup_detrend(initial_smooth=False, initial_rms_mask=False, initial_rms_threshold=3,
                               initial_rms_bin_hours=9, n_detrends=2, detrend_method="gp", cores=3,
                               auto_detrend_periodic_signals=True, auto_detrend_ratio=1 / 2,
                               auto_detrend_method="cosine")
        self.assertEqual(False, sherlock.initial_smooth)
        self.assertEqual(False, sherlock.initial_rms_mask)
        self.assertEqual(3, sherlock.initial_rms_threshold)
        self.assertEqual(9, sherlock.initial_rms_bin_hours)
        self.assertEqual(2, sherlock.n_detrends)
        self.assertEqual("gp", sherlock.detrend_method)
        self.assertEqual(3, sherlock.detrend_cores)
        self.assertEqual(True, sherlock.auto_detrend_periodic_signals)
        self.assertEqual(1 / 2, sherlock.auto_detrend_ratio)
        self.assertEqual("cosine", sherlock.auto_detrend_method)

    def test_setup_transit_adjust_params(self):
        sherlock = Sherlock(False, None)
        sherlock.setup_transit_adjust_params(max_runs=5, period_protec=5, period_min=1, period_max=2, bin_minutes=5,
                                             run_cores=3, snr_min=6, sde_min=5, fap_max=0.05, mask_mode="subtract",
                                             best_signal_algorithm="quorum", quorum_strength=2)
        self.assertEqual(5, sherlock.max_runs)
        self.assertEqual(5, sherlock.period_protec)
        self.assertEqual(1, sherlock.period_min)
        self.assertEqual(2, sherlock.period_max)
        self.assertEqual(5, sherlock.bin_minutes)
        self.assertEqual(3, sherlock.run_cores)
        self.assertEqual(6, sherlock.snr_min)
        self.assertEqual(5, sherlock.sde_min)
        self.assertEqual(0.05, sherlock.fap_max)
        self.assertEqual("subtract", sherlock.mask_mode)
        self.assertEqual("quorum", sherlock.best_signal_algorithm)
        # TODO test quorum strength

    def test_scoring_algorithm(self):
        sherlock = Sherlock(False, None)
        sherlock.setup_transit_adjust_params(best_signal_algorithm="basic")
        self.assertTrue(isinstance(sherlock.signal_score_selectors[sherlock.best_signal_algorithm],
                                   BasicSignalSelector))
        sherlock.setup_transit_adjust_params(best_signal_algorithm="border-correct")
        self.assertTrue(isinstance(sherlock.signal_score_selectors[sherlock.best_signal_algorithm],
                                   SnrBorderCorrectedSignalSelector))
        sherlock.setup_transit_adjust_params(best_signal_algorithm="quorum")
        self.assertTrue(isinstance(sherlock.signal_score_selectors[sherlock.best_signal_algorithm],
                                   QuorumSnrBorderCorrectedSignalSelector))

    def test_preparer(self):
        object_info = MissionObjectInfo("TIC 1234567", 'all')
        sherlock = Sherlock(False, object_info)
        self.assertTrue(isinstance(sherlock.lightcurve_builders[type(object_info)], MissionLightcurveBuilder))
        object_info = MissionFfiIdObjectInfo("TIC 1234567", 'all')
        sherlock = Sherlock(False, object_info)
        self.assertTrue(isinstance(sherlock.lightcurve_builders[type(object_info)], MissionFfiLightcurveBuilder))
        object_info = MissionFfiCoordsObjectInfo(19, 15, 'all')
        sherlock = Sherlock(False, object_info)
        self.assertTrue(isinstance(sherlock.lightcurve_builders[type(object_info)], MissionFfiLightcurveBuilder))
        object_info = MissionInputObjectInfo("TIC 1234567", "testfilename.csv")
        sherlock = Sherlock(False, object_info)
        self.assertTrue(isinstance(sherlock.lightcurve_builders[type(object_info)], MissionInputLightcurveBuilder))
        object_info = InputObjectInfo("testfilename.csv")
        sherlock = Sherlock(False, object_info)
        self.assertTrue(isinstance(sherlock.lightcurve_builders[type(object_info)], MissionInputLightcurveBuilder))

    def test_refresh_tois(self):
        sherlock = Sherlock(False, None)
        sherlock.ois_manager.update_tic_csvs()
        try:
            self.assertTrue(os.path.isfile(sherlock.ois_manager.tois_csv))
        finally:
            os.remove(sherlock.ois_manager.tois_csv)

    def test_refresh_kois(self):
        sherlock = Sherlock(False, None)
        sherlock.ois_manager.update_kic_csvs()
        try:
            self.assertTrue(os.path.isfile(sherlock.ois_manager.kic_star_csv))
        finally:
            os.remove(sherlock.ois_manager.kic_star_csv)
        try:
            self.assertTrue(os.path.isfile(sherlock.ois_manager.kois_csv))
        finally:
            os.remove(sherlock.ois_manager.kois_csv)

    def test_refresh_epicois(self):
        sherlock = Sherlock(False, None)
        sherlock.ois_manager.update_epic_csvs()
        try:
            self.assertTrue(os.path.isfile(sherlock.ois_manager.epic_csv))
        finally:
            os.remove(sherlock.ois_manager.epic_csv)

    def test_ois_loaded(self):
        sherlock = Sherlock(False, None)
        sherlock.load_ois(True)
        sherlock.filter_hj_ois()
        try:
            self.assertGreater(len(sherlock.ois.index), 100)
            sherlock.limit_ois(0, 5)
            self.assertEqual(len(sherlock.ois.index), 5)
            self.assertTrue(sherlock.use_ois)
        finally:
            os.remove(sherlock.ois_manager.tois_csv)
            os.remove(sherlock.ois_manager.ctois_csv)
            os.remove(sherlock.ois_manager.kic_star_csv)
            os.remove(sherlock.ois_manager.kois_csv)
            os.remove(sherlock.ois_manager.epic_csv)

    def test_run_empty(self):
        sherlock = Sherlock(False, [])
        self.assertFalse(sherlock.use_ois)
        sherlock.run()
        object_dir = "TIC181084752_FFI_all"
        self.assertFalse(os.path.exists(object_dir))

    def test_run(self):
        sherlock = Sherlock(False, [MissionFfiIdObjectInfo("TIC 181804752", 'all')])
        sherlock.setup_detrend(n_detrends=1, initial_smooth=False, initial_rms_mask=False)\
            .setup_transit_adjust_params(max_runs=1).run()
        self.__assert_run_files("TIC181804752_FFI_all", assert_rms_mask=False)

    def test_run_with_rms_mask(self):
        sherlock = Sherlock(False, [MissionFfiIdObjectInfo("TIC 181804752", 'all')])
        sherlock.setup_detrend(n_detrends=1, initial_rms_mask=True) \
            .setup_transit_adjust_params(max_runs=1).run()
        self.__assert_run_files("TIC181804752_FFI_all")

    def test_run_with_explore(self):
        sherlock = Sherlock(False, [MissionFfiIdObjectInfo("TIC 181804752", 'all')], True)
        sherlock.setup_detrend(n_detrends=1, initial_rms_mask=True) \
            .setup_transit_adjust_params(max_runs=1).run()
        run_dir = "TIC181804752_FFI_all"
        self.assertTrue(os.path.exists(run_dir))
        self.assertTrue(os.path.exists(run_dir + "/Periodogram_TIC181804752_FFI_all.png"))
        self.assertFalse(os.path.exists(run_dir + "/1"))

    def test_run_with_star_info(self):
        sherlock = Sherlock(False, [MissionFfiIdObjectInfo("TIC 181804752", 'all',
                                                           star_info=StarInfo(ld_coefficients=(0.15,0.25),
                                                                              teff=4000,
                                                                              lum=1.50, logg=0.15, radius=0.4,
                                                                              radius_min=0.10, radius_max=0.15,
                                                                              mass=0.3, mass_min=0.05, mass_max=0.075,
                                                                              ra=15, dec=87))], True)
        sherlock.setup_detrend(n_detrends=1, initial_rms_mask=True)\
            .setup_transit_adjust_params(max_runs=1).run()
        run_dir = "TIC181804752_FFI_all"
        self.assertTrue(os.path.exists(run_dir))
        self.assertTrue(os.path.exists(run_dir + "/Periodogram_TIC181804752_FFI_all.png"))
        self.assertFalse(os.path.exists(run_dir + "/1"))
        with open(run_dir + '/TIC181804752_FFI_all_report.log') as f:
            content = f.read()
            self.assertTrue('mass = 0.3' in content)
            self.assertTrue('mass_min = 0.25' in content)
            self.assertTrue('mass_max = 0.375' in content)
            self.assertTrue('radius = 0.4' in content)
            self.assertTrue('radius_min = 0.3' in content)
            self.assertTrue('radius_max = 0.55' in content)
            self.assertTrue('limb-darkening estimates using quadratic LD (a,b)= (0.15, 0.25)' in content)
            self.assertTrue('teff = 4000' in content)
            self.assertTrue('logg = 0.15' in content)
            self.assertTrue('lum = 1.50' in content)

    def test_run_with_transit_customs(self):
        sherlock = Sherlock(False, [MissionFfiIdObjectInfo("TIC 181804752", 'all')], False)
        sherlock.setup_detrend(n_detrends=1, initial_rms_mask=True)\
            .setup_transit_adjust_params(max_runs=1, oversampling=5.5, t0_fit_margin=0.09, duration_grid_step=1.075,
                                         fit_method="bls", best_signal_algorithm="quorum", quorum_strength=0.31)\
            .run()
        run_dir = "TIC181804752_FFI_all"
        with open(run_dir + '/TIC181804752_FFI_all_report.log') as f:
            content = f.read()
            self.assertTrue('Fit method: box' in content)
            self.assertTrue('Duration step: 1.075' in content)
            self.assertTrue('T0 Fit Margin: 0.09' in content)
            self.assertTrue('Oversampling: 5' in content)
            self.assertTrue('Signal scoring algorithm: quorum' in content)
            self.assertTrue('Quorum algorithm vote strength: 0.31' in content)
        self.__assert_run_files(run_dir)

    def __assert_run_files(self, object_dir, assert_rms_mask=True):
        run_dir = object_dir + "/1"
        periodogram_file = object_dir + "/Periodogram_TIC181804752_FFI_all.png"
        rms_mask_file = object_dir + "/High_RMS_Mask_TIC181804752_FFI_all.png"
        lc_file = object_dir + "/lc.csv"
        report_file = object_dir + "/TIC181804752_FFI_all_report.log"
        candidates_csv_file = object_dir + "/candidates.csv"
        try:
            self.assertTrue(os.path.exists(run_dir))
            self.assertTrue(os.path.isfile(periodogram_file))
            if assert_rms_mask:
                self.assertTrue(os.path.isfile(rms_mask_file))
            self.assertTrue(os.path.isfile(lc_file))
            self.assertTrue(os.path.isfile(report_file))
            self.assertTrue(os.path.isfile(candidates_csv_file))
        finally:
            shutil.rmtree(object_dir, ignore_errors=True)


if __name__ == '__main__':
    unittest.main()


import re
from abc import ABC, abstractmethod
from sherlockpipe.star.EpicStarCatalog import EpicStarCatalog
from sherlockpipe.star.KicStarCatalog import KicStarCatalog
from sherlockpipe.star.TicStarCatalog import TicStarCatalog


class LightcurveBuilder(ABC):
    OBJECT_ID_REGEX = "^(KIC|TIC|EPIC)[-_ ]([0-9]+)$"
    NUMBERS_REGEX = "[0-9]+$"
    MISSION_ID_KEPLER = "KIC"
    MISSION_ID_KEPLER_2 = "EPIC"
    MISSION_ID_TESS = "TIC"

    def __init__(self):
        self.star_catalogs = {}
        self.star_catalogs[self.MISSION_ID_KEPLER] = KicStarCatalog()
        self.star_catalogs[self.MISSION_ID_KEPLER_2] = EpicStarCatalog()
        self.star_catalogs[self.MISSION_ID_TESS] = TicStarCatalog()

    @abstractmethod
    def build(self, object_info, sherlock_dir):
        pass

    def parse_object_id(self, object_id):
        object_id_parsed = re.search(self.OBJECT_ID_REGEX, object_id)
        mission_prefix = object_id[object_id_parsed.regs[1][0]:object_id_parsed.regs[1][1]]
        id = object_id[object_id_parsed.regs[2][0]:object_id_parsed.regs[2][1]]
        if mission_prefix == self.MISSION_ID_KEPLER:
            mission = "Kepler"
        elif mission_prefix == self.MISSION_ID_KEPLER_2:
            mission = "K2"
        elif mission_prefix == self.MISSION_ID_TESS:
            mission = "TESS"
        else:
            raise ValueError("Invalid object id " + object_id)
        return mission, mission_prefix, int(id)


import os.path
from ..ffi import ffi

def test_ffi_dir():
    """Are FFIs downloaded to the correct location?"""

    # set default download_dir to `~/.eleanor/sector_{}/ffis`
    default_dir = os.path.join(os.path.expanduser('~'), '.eleanor',
                               'sector_{}'.format(1), 'ffis')

    # create an ffi and fetch the download dir
    test_ffi = ffi(sector=1)
    ffi_dir = test_ffi._fetch_ffi_dir()

    # make sure the ffi_dir exists and matches the default dir
    assert(os.path.isdir(ffi_dir))
    assert(ffi_dir == default_dir)



'''
filter.py: some simple routines to filter time-series.
'''
'''
norm.py: NaN-robust wrappers for mean, median, min, max... + MAD
scatter estimates + simple normalisation routines
'''

import numpy as np
from scipy import signal, interpolate

def medsig(array):
    '''
    Return median and outlier-robust estimate of standard deviation
    (1.48 x median of absolute deviations).
    '''
    l = np.isfinite(array)
    if sum(l) == 0:
        return np.nan, np.nan
    if sum(l) == 1:
        return array[l], np.nan
    med = np.median(array[l])
    sig = 1.48 * np.median(abs(array[l] - med))
    return med, sig
  
def mysum(array, axis = None):
    '''
    Return sum of array along specified axis (ignoring NaNs)
    '''
    arr = np.copy(array)
    l = np.isfinite(arr)
    if l.any() == False: return np.nan
    arr[l==False] = 0.0
    res = np.sum(arr, axis = axis)
    if axis != None:
        l_sum = np.sum(l, axis = axis)
        res[l_sum==0] = np.nan
    return res

def mymean(array):
    '''
    Return mean of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].mean())

def mystd(array):
    '''
    Return standard deviation of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].std())

def mymin(array):
    '''
    Return minimum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].min())

def mymax(array):
    '''
    Return maximum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return np.nan
    return(array[l].max())

def myargmin(array):
    '''
    Return index of minimum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return -1
    return(array[l].argmin())

def myargmax(array):
    '''
    Return index maximum of array (ignoring NaNs)
    '''
    l = np.isfinite(array)
    if l.any() == False: return -1
    return(array[l].argmax())

def myabs(array):
    l = np.isfinite(array)
    a = np.copy(array)
    a[l] = abs(array[l])
    return a

def norm01(array):
    '''
    Normalize array to zero mean and unit variance
    '''
    med, sig = medsig(array)
    a = np.copy(array)
    a -= med
    a /= sig
    return a, med, sig

def unnorm01(array, med, sig):
    '''
    Unnormalize array
    '''
    a = np.copy(array) 
    a *= sig
    a += med
    return a

def scale01(array):
    '''
    Scale array to range from 0 to 1
    '''
    nmeas, nobj = np.shape(array)
    arr = np.zeros((nmeas, nobj)) + np.nan
    mins = np.zeros(nobj)
    maxs = np.zeros(nobj)
    for i in np.arange(nobj):
        cur = array[:,i]
        mins[i] = mymin(cur)
        maxs[i] = mymax(cur)
        if np.isfinite(mins[i]) * np.isfinite(maxs[i]) * (maxs[i] > mins[i]):
            arr[:,i] = (cur - mins[i]) / (maxs[i] - mins[i])
    return arr, mins, maxs

def unscale(array):
    '''
    Unscale array from 0 to 1 back to original range
    '''
    nmeas, nobj = np.shape(array)
    arr = np.zeros((nmeas, nobj)) + np.nan
    for i in np.arange(nobj):
        arr[:,i] = (array[:,i] * (maxs[i] - mins[i])) + mins[i]
    return arr


# ---------

def filt1d(array, nmed, nlin, fill = False, circ = False, kernel = 'box'):
    '''
    Nonlinear (median+linear) filter with edge reflection.
    The 'kernel' parameter can set to either 'box' (boxcar, default)
    or 'gauss' (Gaussian)
    '''

    nmed = 2 * int(nmed/2) + 1
    N = len(array)
    if N < (3*nmed):
        print('filt1d:')
        print('    Warning: array too short compared to nmed')
        print('    Returning without doing anything')
        return array

    # Check for NaNs
    lnan = np.isnan(array)
    nnan = lnan.sum()
#    print ("nnan")
#    print (nnan)

    if nnan != 0:
        lnotnan = np.where(lnan==0)[0]
        if len(lnotnan) == 0:
            print('filt1d:')
            print('    Warning: no non-NaN data')
            print('    Returning without doing anything')
            return array
        med = np.median(array[lnotnan])
        # Fill in any data gaps by interpolating over them ...
        work = np.zeros(N)
        il = lnotnan.min()
        ih = lnotnan.max()
        xnew = np.arange(ih-il+1) + il
        f = interpolate.interp1d(lnotnan, array[lnotnan])  
        work[il:ih+1] = f(xnew)
        # ... or extending slope of nearest data points if at the edges
        if (il!=0):
            slope = work[il+1] - work[il]
            for i in range(il): work[i] = work[il] - slope*(il-i)
        if (ih!=N-1):
            slope = work[ih] - work[ih-1]
            for i in range(N-ih-1)+ih+1: work[i] = work[ih] + slope*(i-ih)
    else:
#        print ('nonans')
        work = np.copy(array)
    # Edge reflection
    nr = min(20, nlin)
    sz = max([nmed, nlin])
    if sz >= (N-1): 
        sz = N-2
    if circ != False:
        wl = work[N-sz:]
        wr = work[:sz]
    else:
        wl = array[0:sz]
        pivot = np.median(array[:nr])
        wl = 2 * pivot - wl
        wl = wl[::-1] # reverse array
        wr = array[N-sz:N]
        pivot = np.median(array[N-nr:N])
        wr = 2 * pivot - wr
        wr = wr[::-1] # reverse array
    work2 = np.zeros(N + 2 * sz)
    work2[0:sz] = wl
    work2[sz:sz+N] = work
    work2[sz+N:2*sz+N] = wr
    # Filter
    if nmed > 1: work = signal.medfilt(work2, nmed) 
    else: work = work2
    if kernel == 'gauss':
        box = signal.gaussian(10 * nlin, nlin)
    else:
        box = signal.boxcar(2*nlin+1)
    work = signal.convolve(work, box) / float(2*nlin+1)
    padd = int((len(work) - N - 2 * sz) / 2)
    #print(padd,N,sz)
    work = work[padd:padd+N+2*sz]
    # return to orginal array bounds
    result = work[sz:N+sz]
    # replace bad data if present
    if (fill==False) * (nnan!=0): result[lnan] = np.nan
    #print ("reached the end")
    return result

def boxcare(array, box, fill = False, circ = False):
    """Boxcar filter with edge reflection"""
    nlin = int(box / 2)
    return filt1d(array, 1, nlin, fill = fill, circ = circ)

def gausse(array, box, fill = False, circ = False):
    """Gaussian filter with edge reflection"""
    nlin = int(box / 2)
    return filt1d(array, 1, nlin, fill = fill, circ = circ, kernel = Gauss)

def NIF(array, nmed, nlin, nsig = 3.0, prefilt = False, \
            fill = True, circ = False, verbose = False):
    '''
    Non-linear iterative filter with k-sigma clipping (Aigrain & Irwin
    2004). If fill = True, clipped values are filled in by linear
    interpolation.
    '''
    # Pre-filter if requested

    if prefilt != False:
        work = filt1d(np.copy(array), 7, 3) 
    else:
        work = np.copy(array)
    # Start iteration loop
    irejo = -1
    irej = 0
    for i in range(5):
        if i > 0:
            irejo = irej
            out = np.isnan(work) + (myabs(array - work) > (nsig*sigma))
            keep = (out == False)
            irej = sum(out)
            work = np.copy(array)
            if irej != 0: work[out] = np.nan

        keep = np.isfinite(work)
        work[keep] = filt1d(work[keep], nmed, nlin, fill = fill, circ = circ)
        sigma = 1.48 * np.median(myabs(work[keep] - array[keep]))
        #if verbose == True: print('NIF:', i, irej, irejo, sigma)
        if irej <= irejo: break
    return work



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

#::: modules
import numpy as np
import os, sys
import ellc
from transitleastsquares import catalog_info
import astropy.constants as ac
import astropy.units as u
import lightkurve as lk
import pandas as pd


np.random.seed(42)

#::: load data and set the units correctly
TIC_ID = 85400193    # TIC_ID of our candidate
lcf= lk.search_lightcurvefile('TIC '+str(TIC_ID), mission="tess").download_all()
ab, mass, massmin, massmax, radius, radiusmin, radiusmax = catalog_info(TIC_ID=TIC_ID)

#units for ellc
rstar=radius*u.R_sun
mstar=mass*u.M_sun
#mass and radius for the TLS
#rstar=radius
#mstar=mass
mstar_min = mass-massmin
mstar_max = mass+massmax
rstar_min = radius-radiusmin
rstar_max = radius+radiusmax

#uncomment the following lines to check that the parameters used are correct.

#print('\n STELLAR PROPERTIES FOR THE SIGNAL SEARCH')
#print('================================================\n')
#print('limb-darkening estimates using quadratic LD (a,b)=', ab)
#print('mass =', format(mstar,'0.5f'))
#print('mass_min =', format(mstar_min,'0.5f'))
#print('mass_max =', format(mstar_max,'0.5f'))
#print('radius =', format(rstar,'0.5f'))
#print('radius_min =', format(rstar_min,'0.5f'))
#print('radius_max =', format(rstar_max,'0.5f'))


lc=lcf.PDCSAP_FLUX.stitch().remove_nans() # remove of the nans
lc_new=lk.LightCurve(time=lc.time, flux=lc.flux,flux_err=lc.flux_err)
clean=lc_new.remove_outliers(sigma_lower=float('inf'), sigma_upper=3) #remove outliers over 3sigma
flux0=clean.flux
time=clean.time
flux_err = clean.flux_err
#period_maximum=(max(time)-min(time))/2.
#time, flux0 = np.genfromtxt('TESS_phot.csv', delimiter=',', unpack=True)
#rstar = 0.211257 * 41.46650444642 #in Rearth

#::: make model        
def make_model(epoch, period, rplanet):
    #a = (7.495e-6 * period**2)**(1./3.)*u.au #in AU
    P1=period*u.day
    a = np.cbrt((ac.G*mstar*P1**2)/(4*np.pi**2)).to(u.au)
    #print("radius_1 =", rstar.to(u.au) / a) #star radius convert from AU to in units of a 
    #print("radius_2 =", rplanet.to(u.au) / a)
    texpo=2./60./24.
    #print("T_expo = ", texpo,"dy")
    #tdur=t14(R_s=radius, M_s=mass,P=period,small_planet=False) #we define the typical duration of a small planet in this star
    #print("transit_duration= ", tdur*24*60,"min" )
    model = ellc.lc(
           t_obs = time,
           radius_1 = rstar.to(u.au) / a, #star radius convert from AU to in units of a
           radius_2 = rplanet.to(u.au) / a, #convert from Rearth (equatorial) into AU and then into units of a
           sbratio = 0,
           incl = 90,
           light_3 = 0,
           t_zero = epoch,
           period = period,
           a = None,
           q = 1e-6,
           f_c = None, f_s = None,
           ldc_1=[0.2755,0.5493], ldc_2 = None,
           gdc_1 = None, gdc_2 = None,
           didt = None,
           domdt = None,
           rotfac_1 = 1, rotfac_2 = 1,
           hf_1 = 1.5, hf_2 = 1.5,
           bfac_1 = None, bfac_2 = None,
           heat_1 = None, heat_2 = None,
           lambda_1 = None, lambda_2 = None,
           vsini_1 = None, vsini_2 = None,
           t_exp=texpo, n_int=None,
           grid_1='default', grid_2='default',
           ld_1='quad', ld_2=None,
           shape_1='sphere', shape_2='sphere',
           spots_1=None, spots_2=None,
           exact_grav=False, verbose=1)

    flux_t = flux0 + model - 1.
    if model[0] > 0:
        flux = flux_t
        flux_err_model = flux_err
        time_custom = time
    else:
        flux = []
        time_custom = []
        flux_err_model = []
    return time_custom, flux, flux_err_model
    #minutes=10
    #print(len(time))
    #print(min(time),max(time))
    #bins=len(time)*2./minutes
    #print(bins)
    #bin_means, bin_edges, binnumber = stats.binned_statistic(time, flux, statistic='mean', bins=bins)
    #bin_stds, _, _ = stats.binned_statistic(time, flux, statistic='std', bins=bins)
    #bin_width = (bin_edges[1] - bin_edges[0])
    #bin_centers = bin_edges[1:] - bin_width/2
    #print('RMS PDCSAP flux (ppm): ',np.std(flux0[~np.isnan(flux0)])*1e6)
    #print('RMS model (ppm): ',np.std(flux[~np.isnan(flux)])*1e6)
    #print('RMS 10min bin detrended (ppm): ',np.std(bin_means[~np.isnan(bin_means)])*1e6)
    
    #fig, (ax1,ax2,ax3) = plt.subplots(3, 1, figsize=(10,5), constrained_layout=True)
    ##ax1
    #ax1.plot(time, flux0, linewidth=0.05 ,color='black', alpha=0.4)
    ##ax1.legend(bbox_to_anchor=(0.85, 0.95), loc=2, borderaxespad=0.,fontsize=8)
    #ax1.set_ylabel("Normalized flux")
    #ax1.set_xlim(1766,1769)
    ##ax2
    #ax2.plot(time, flux0, linewidth=0.05 ,color='black', alpha=0.4)
    ##ax2.plot(time, model, linewidth=0.9 ,color='firebrick', alpha=1)
    #ax2.errorbar(time, model, marker='.', markersize=2, color='firebrick', alpha=1, linestyle='none')
    #ax2.set_ylabel("Normalized flux")
    #ax2.set_xlim(1766,1769)
    ##ax3
    #ax3.plot(time, flux, linewidth=0.1 ,color='teal', alpha=0.5)
    #ax3.errorbar(bin_centers, bin_means, marker='.', markersize=4, color='darkorange', alpha=1, linestyle='none')
    #ax3.set_ylabel("Normalized flux")
    #ax3.set_xlabel("Time (days)")
    #ax3.set_xlim(1766,1769)
    #plt.savefig('model.png', dpi=200)



def logprint(*text):
#    print(*text)
    original = sys.stdout
    with open( os.path.join('tls/'+'P = '+str(period)+' days, Rp = '+str(rplanet)+'.log'), 'a' ) as f:
        sys.stdout = f
        print(*text)
    sys.stdout = original

    
#::: iterate through grid of periods and rplanet
dir = "/home/pozuelos/martin/curves"
if not os.path.isdir(dir):
    os.mkdir(dir)
max_period = 10
min_period = 0.5
for period in np.arange(min_period, max_period, 0.5):
    for t0 in np.arange(time[60], time[60] + period - 0.1, period / 5):
        for rplanet in np.arange(4, 0.65, -0.1):
            rplanet = np.around(rplanet, decimals=2)*u.R_earth
            print('\n')
            print('P = '+str(period)+' days, Rp = '+str(rplanet) + ", T0 = " + str(t0))
            time_model, flux_model, flux_err_model = make_model(t0, period, rplanet)
            file_name = os.path.join(dir + '/P' + str(period) + '_R' + str(rplanet.value) + '_' + str(t0) + '.csv')
            lc_df = pd.DataFrame(columns=['#time', 'flux', 'flux_err'])
            lc_df['#time'] = time_model
            lc_df['flux'] = flux_model
            lc_df['flux_err'] = flux_err_model
            lc_df.to_csv(file_name, index=False)



from contextlib import contextmanager
from timeit import default_timer
from sherlockpipe.sherlock import Sherlock
from sherlockpipe.objectinfo.InputObjectInfo import InputObjectInfo
from sherlockpipe.objectinfo.MissionFfiCoordsObjectInfo import MissionFfiCoordsObjectInfo
from sherlockpipe.objectinfo.MissionFfiIdObjectInfo import MissionFfiIdObjectInfo
from sherlockpipe.objectinfo.MissionInputObjectInfo import MissionInputObjectInfo
from sherlockpipe.objectinfo.MissionObjectInfo import MissionObjectInfo


@contextmanager
def elapsed_timer():
    start = default_timer()
    elapser = lambda: str(default_timer() - start)
    yield lambda: elapser()
    end = default_timer()
    elapser = lambda: str(end - start)


with elapsed_timer() as elapsed:
    # We will use only one mission object and set the period to be initially detrended with a window size of 1/3 times
    # the given period.
    sherlock = Sherlock([MissionObjectInfo("TIC 181804752", 'all')])\
        .setup_detrend(auto_detrend_ratio=1 / 3).run()
    print("Analysis took " + elapsed() + "s")



from contextlib import contextmanager
from timeit import default_timer
from sherlockpipe.sherlock import Sherlock
from sherlockpipe.objectinfo.MissionObjectInfo import MissionObjectInfo


@contextmanager
def elapsed_timer():
    start = default_timer()
    elapser = lambda: str(default_timer() - start)
    yield lambda: elapser()
    end = default_timer()
    elapser = lambda: str(end - start)


with elapsed_timer() as elapsed:
    # We will use only one object id so we can explain better the detrend configs that the coder can select
    # We will:
    # 1 Set the maximum number of runs to be executed.
    # 2 Select the period protect value, which TODO
    # 3 Select the min period for a transit to be fit.
    # 4 Select the max period for a transit to be fit.
    # 5 Select the binning for TODO
    # 6 Select the number of CPU cores to be used for the transit search.
    # 7 Select the min SNR, the min SDE and the max FAP to stop the runs execution for each object.
    # 8 Select the found transits masking method. We use subtract here as example, but it is discouraged.
    # 9 Select the best signal algorithm, which provides a different implementation to decide which of the detrend
    # signals is the stronger one to be selected.
    # 10 Set the strength of the quorum algorithm votes, which makes every vote that is found to increase the SNR by
    # a factor of 1.2 for our selection.
    sherlock = Sherlock([MissionObjectInfo("TIC 181804752", 'all')])\
        .setup_transit_adjust_params(max_runs=10, period_protec=12, period_min=1, period_max=10, bin_minutes=20,
                                 run_cores=3, snr_min=6, sde_min=6, fap_max=0.08, mask_mode="subtract",
                                 best_signal_algorithm='quorum', quorum_strength=1.2)\
        .run()
    print("Analysis took " + elapsed() + "s")




# -----------------------------
'''
This code makes sure that everytime that the returned data is what it is expected to be. 

This test case is carried out with a pre-downloaded lighcurve file from TIC 55525572 Sector 5.


run using: python -m unittest tests/test_data_products.py

'''

# -------------


import os
import mock
import unittest
import requests
import argparse
from argparse import ArgumentParser

import warnings
warnings.filterwarnings("ignore")

import sys
import LATTE.LATTEutils as utils

# test the downloading of the data
# get the outir (or indir) path
syspath = str(os.path.abspath(utils.__file__))[0:-14]


indir = "./LATTE/output"        

# test with these input parameters (this data is already downloaded so it's only testing that the data extraction from the FITS files works)
tic = '55525572'
sector = '5'
sectors = [5]
transit_list = [1454.7]
transit_sec = '5'
# -------------

class TestDataImport_LC(unittest.TestCase):
    '''
    Test the extraction of the information from the LC file (which is already on the system)
    '''

    def test_LC_request_response(self):
        # Call the service, which will send a request to the server.
        alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad = utils.download_data(indir,sector, tic, binfac = 5, test = './LATTE/tests/tic55525572_lc.fits')

        # If the request is sent successfully, then I expect a response to be returned.
        self.assertAlmostEqual(float(alltime[100]),       float(1438.1174094083246), places=5, msg='LC data is not what it should be.')
        self.assertAlmostEqual(float(allflux[100]),       float(1.0003796815872192), places=5)
        self.assertAlmostEqual(float(allflux_err[100]),   float(0.0011117355898022652), places=5)
        self.assertAlmostEqual(float(all_md[0]),          float(1441.0243360822994), places=5)
        self.assertAlmostEqual(float(alltimebinned[100]), float(1438.6757392292352), places=5)
        self.assertAlmostEqual(float(allfluxbinned[100]), float(0.9995232224464417), places=5)
        self.assertAlmostEqual(float(allx1[100]),         float(-0.010814569022272735), places=5)
        self.assertAlmostEqual(float(allx2[100]),         float(-0.011804798617959023), places=5)
        self.assertAlmostEqual(float(ally1[100]),         float(-0.024266568269581512), places=5)
        self.assertAlmostEqual(float(ally2[100]),         float(-0.02981671877205372), places=5)
        self.assertAlmostEqual(float(alltime12[100]),     float(1438.1312982026025), places=5)


class TestDataImport_TP(unittest.TestCase):
    '''
    Test the extraction of the information from the TP file (which is already on the system)
    '''

    def test_LC_request_response(self):

        # Call the service, which will send a request to the server.
        X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list, t_list, T0_list, tpf_filt_list = utils.download_tpf_mast(indir, transit_sec, transit_list, tic, test = './LATTE/tests/tic55525572_tp.fits')

        # If the request is sent successfully, then I expect a response to be returned.
        self.assertAlmostEqual(float(X1_list[0][0][0]),float(23.402481079101562), places=5, msg ='TP data is not what it should be.')
        self.assertAlmostEqual(float(oot_list[0][0]),float(0.0))
        self.assertAlmostEqual(float(intr_list[0][0]),float(0.0))
        self.assertAlmostEqual(float(bkg_list[0][0][0]),float(29.239688873291016), places=5)
        self.assertAlmostEqual(float(apmask_list[0][0][0]),float(0.0))
        self.assertAlmostEqual(float(arrshape_list[0][0]),float(18944.0))
        self.assertAlmostEqual(float(t_list[0][0]),float(1437.9924102871835), places=5,)
        self.assertAlmostEqual(float(T0_list[0]),float(1454.7))


class TestDataImport_TP_lighkurve(unittest.TestCase):
    '''
    Test the extraction of the information from the TP file (which is already on the system)
    '''

    def test_LC_request_response(self):

        # Call the service, which will send a request to the server.
        TESS_unbinned_t_l, TESS_binned_t_l, small_binned_t_l, TESS_unbinned_l, TESS_binned_l, small_binned_l, tpf_list = utils.download_tpf_lightkurve(indir, transit_list, sectors, tic, test = './LATTE/tests/tic55525572_tp.fits')
        
        print(float(TESS_unbinned_t_l[0]))
        print(float(TESS_binned_t_l[0]))
        print(float(small_binned_t_l[0]))
        print(float(TESS_unbinned_l[0]))
        print(float(TESS_binned_l[0]))
        print(float(small_binned_l[0]))

        
        # If the request is sent successfully, then I expect a response to be returned.
        self.assertAlmostEqual(float(TESS_unbinned_t_l[0]), float(1437.9924102871835), msg='TP lighkurve data is not what it should be.')
        self.assertAlmostEqual(float(TESS_binned_t_l[0]), float(1437.9972713631325), places=5)
        self.assertAlmostEqual(float(small_binned_t_l[0]), float(1437.9972713631325), places=5)
        self.assertAlmostEqual(float(TESS_unbinned_l[0]), float(0.9975904130977179), places=5)
        self.assertAlmostEqual(float(TESS_binned_l[0]), float(0.9986087992155438), places=5)
        self.assertAlmostEqual(float(small_binned_l[0]), float(0.9971978343909532), places=5)



# ---------------------
if __name__ == "__main__":

    unittest.main()




import math
from sherlockpipe.search_zones.SearchZone import SearchZone
from sherlockpipe.star.HabitabilityCalculator import HabitabilityCalculator
from sherlockpipe.star.starinfo import StarInfo


class HabitableSearchZone(SearchZone):
    """
    Implementation for calculating minimum and maximum search periods for the habitable zone of an input star.
    """
    def __init__(self):
        super().__init__()

    def calculate_period_range(self, star_info: StarInfo):
        habitability_calc = HabitabilityCalculator()
        hz_periods = habitability_calc.calculate_hz_periods(star_info.teff, star_info.lum, star_info.mass)
        if hz_periods is None:
            return None
        return hz_periods[1], hz_periods[2]



import numpy as np

'''Calculates the Habitability Zone of a star based on the Kopparapu et al (2003) equations. These equations are only
valid for stars between 2600 K < Teff < 7200 K.'''
class HabitabilityCalculator:
    sun_luminosity = 1.
    G = 6.674e-11  # m3 kg-1 s-2
    ##
    ## Stellar flux coefficients (table 3 from kopparapu et al 2013)
    ##
    e_end = 1.00
    Nout = 10000
    e = np.linspace(0.0001, e_end, Nout, endpoint=False)

    def __init__(self):
        pass

    # equation (2) from kopparapu et al 2013, for non ecentric orbits
    def __seff(self, Seff_sun, a, b, c, d, t_star):
        Seff = Seff_sun + a * t_star + b * t_star ** 2 + c * t_star ** 3 + d * t_star ** 4
        return Seff

    # equation (3) from kopparapu et al 2013, for non ecentric orbits
    def __dis(self, Seff, luminosity):
        return (luminosity / self.sun_luminosity / Seff) ** 0.5

    # having equations (3) y (4) for eccentric orbits from kopparapu et al 2013
    def __seff_prime(self, A, e):
        return A / np.sqrt(1. - e ** 2)

    def calculate_hz(self, t_eff, luminosity):
        """
        Calculates the habitable zone ranges in astronomical units
        @param t_eff: the stellar effective temperature
        @param luminosity: the stellar luminosity in sun luminosities
        @return: a tuple of size 4 with the recent venus, moist greenhouse, maximum greenhouse and early mars orbital
        semi-major axises for the given star parameters.
        """
        if t_eff < 2600 or t_eff > 7200:
            return None
        t_star = t_eff - 5700
        # Recent Venus
        s_eff_rv = 1.7763
        a_rv = 1.4335e-4
        b_rv = 3.3954e-9
        c_rv = -7.6364e-12
        d_rv = -1.1950e-15
        Seff_rv = self.__seff(s_eff_rv, a_rv, b_rv, c_rv, d_rv, t_star)
        Dis_rv = self.__dis(Seff_rv, luminosity)
        Seff_prime_rv = self.__seff_prime(Seff_rv, self.e)
        Dis_prime_rv = self.__dis(Seff_prime_rv, t_star)
        # Runaway Greenhouse
        s_eff_rg = 1.0385
        a_rg = 1.2456e-4
        b_rg = 1.4612e-8
        c_rg = -7.6345e-12
        d_rg = -1.7511e-15
        Seff_rg = self.__seff(s_eff_rg, a_rg, b_rg, c_rg, d_rg, t_star)
        Dis_rg = self.__dis(Seff_rg, luminosity)
        Seff_prime_rg = self.__seff_prime(Seff_rg, self.e)
        Dis_prime_rg = self.__dis(Seff_prime_rg, luminosity)
        # Moist Greenhouse
        s_eff_mog = 1.0146
        a_mog = 8.1884e-5
        b_mog = 1.9394e-9
        c_mog = -4.3618e-12
        d_mog = -6.8260e-16
        Seff_mog = self.__seff(s_eff_mog, a_mog, b_mog, c_mog, d_mog, t_star)
        Dis_mog = self.__dis(Seff_mog, luminosity)
        Seff_prime_mog = self.__seff_prime(Seff_mog, self.e)
        Dis_prime_mog = self.__dis(Seff_prime_mog, luminosity)
        # Maximun Greenhouse
        s_eff_mag = 0.3507
        a_mag = 5.9578e-5
        b_mag = 1.6707e-9
        c_mag = -3.0058e-12
        d_mag = -5.1925e-16
        Seff_mag = self.__seff(s_eff_mag, a_mag, b_mag, c_mag, d_mag, t_star)
        Dis_mag = self.__dis(Seff_mag, luminosity)
        Seff_prime_mag = self.__seff_prime(Seff_mag, self.e)
        Dis_prime_mag = self.__dis(Seff_prime_mag, luminosity)
        # Early Mars
        s_eff_em = 0.3207
        a_em = 5.4471e-5
        b_em = 1.5275e-9
        c_em = -2.1709e-12
        d_em = -3.8282e-16
        Seff_em = self.__seff(s_eff_em, a_em, b_em, c_em, d_em, t_star)
        Dis_em = self.__dis(Seff_em, luminosity)
        Seff_prime_em = self.__seff_prime(Seff_em, self.e)
        Dis_prime_em = self.__dis(Seff_prime_em, luminosity)
        return [Dis_rv, Dis_mog, Dis_mag, Dis_em]

    def calculate_hz_periods(self, t_eff, luminosity, mass):
        """
        Calculates the habitable zone ranges in periods
        @param t_eff: the stellar effective temperature
        @param luminosity: the stellar luminosity in sun luminosities
        @param mass: the stellar mass in sun masses
        @return: a tuple of size 4 with the recent venus, moist greenhouse, maximum greenhouse and early mars orbital
        periods for the given star parameters.
        """
        aus = self.calculate_hz(t_eff, luminosity)
        if aus is None:
            return None
        return [self.au_to_period(mass, au) for au in aus]

    def au_to_period(self, mass, au):
        """
        Calculates the orbital period for the semi-major axis assuming a circular orbit.
        @param mass: the stellar mass
        @param au: the semi-major axis in astronomical units.
        @return: the period in days
        """
        mass_kg = mass * 2.e30
        a = au * 1.496e11
        return ((a ** 3) * 4 * (np.pi ** 2) / self.G / mass_kg) ** (1. / 2.) / 3600 / 24

    def calculate_semi_major_axis(self, period, star_mass):
        period_seconds = period * 24. * 3600.
        mass_kg = star_mass * 2.e30
        a1 = (self.G * mass_kg*period_seconds ** 2/4. / (np.pi ** 2)) ** (1. / 3.)
        return a1 / 1.496e11

    def calculate_hz_score(self, t_eff, star_mass, luminosity, period):
        """
        Returns the semi-major axis and the HZ Area [I=Inner, HZ-IO=Habitable Zone (Inner Optimistic),
        HZ=Habitable Zone, HZ-OO=Habitable Zone (Outer Optimistic)
        @param t_eff: the star effective temperature
        @param star_mass: the star mass in sun masses
        @param luminosity: the star luminosity in sun luminosities
        @param period: the period to guess the semi-major axis
        @return: a tuple of semi-major axis and hz position string.
        """
        hz = self.calculate_hz(t_eff, luminosity) if t_eff is not None and not np.isnan(t_eff) else None
        a1_au = self.calculate_semi_major_axis(period, star_mass) if star_mass is not None and not np.isnan(star_mass)\
                else np.nan
        if np.isnan(a1_au) or hz is None:
            hz_position = "-"
        elif a1_au < hz[0]:
            hz_position = 'I'
        elif a1_au >= hz[0] and a1_au < hz[1]:
            hz_position = 'HZ-IO'
        elif a1_au >= hz[1] and a1_au < hz[2]:
            hz_position = 'HZ'
        elif a1_au >= hz[2] and a1_au < hz[3]:
            hz_position = 'HZ-OC'
        else:
            hz_position = 'O'
        return a1_au, hz_position



import pickle
import sys

from sherlockpipe.star.starinfo import StarInfo

from sherlockpipe import sherlock
from sherlockpipe.objectinfo.InputObjectInfo import InputObjectInfo
from sherlockpipe.objectinfo.MissionFfiIdObjectInfo import MissionFfiIdObjectInfo
from sherlockpipe.objectinfo.MissionFfiCoordsObjectInfo import MissionFfiCoordsObjectInfo
from sherlockpipe.objectinfo.MissionInputObjectInfo import MissionInputObjectInfo
from sherlockpipe.objectinfo.MissionObjectInfo import MissionObjectInfo
from argparse import ArgumentParser
from os import path
import yaml
import importlib.util


def load_module(module_path):
    spec = importlib.util.spec_from_file_location("customs", module_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    sys.modules[spec.name] = module
    spec.loader.exec_module(module)
    return module

def get_star_info(properties, id):
    input_star_info = None
    if properties["STAR"] is not None and properties["STAR"][id] is not None:
        star_properties = properties["STAR"][id]
        input_star_info = StarInfo(id, tuple(star_properties["LD_COEFFICIENTS"]) if "LD_COEFFICIENTS" in star_properties else None,
                             star_properties["TEFF"] if "TEFF" in star_properties else None,
                             star_properties["LUM"] if "LUM" in star_properties else None,
                             star_properties["LOGG"] if "LOGG" in star_properties else None,
                             star_properties["RADIUS"] if "RADIUS" in star_properties else None,
                             star_properties["RADIUS_LOWER_ERROR"] if "RADIUS_LOWER_ERROR" in star_properties else None,
                             star_properties["RADIUS_UPPER_ERROR"] if "RADIUS_UPPER_ERROR" in star_properties else None,
                             star_properties["MASS"] if "MASS" in star_properties else None,
                             star_properties["MASS_LOWER_ERROR"] if "MASS_LOWER_ERROR" in star_properties else None,
                             star_properties["MASS_UPPER_ERROR"] if "MASS_UPPER_ERROR" in star_properties else None,
                             star_properties["RA"] if "RA" in star_properties else None,
                             star_properties["DEC"] if "DEC" in star_properties else None)
    return input_star_info

def get_aperture(properties, id):
    input_aperture_file = None
    if properties["APERTURE"] is not None and properties["APERTURE"][id] is not None:
        input_aperture_file = properties["APERTURE"][id]
    return input_aperture_file


if __name__ == '__main__':
    ap = ArgumentParser(description='Searching for Hints of Exoplanets fRom Lightcurves Of spaCe-based seeKers')
    ap.add_argument('--properties', help="Additional properties to be loaded into Sherlock run ", required=True)
    ap.add_argument('--explore', dest='explore', action='store_true', help="Whether to run using mcmc or ns. Default is ns.")
    args = ap.parse_args()
    resources_dir = path.join(path.dirname(__file__))
    file_dir = resources_dir + "/" + 'properties.yaml' if resources_dir != "" and resources_dir is not None \
        else 'properties.yaml'
    print(resources_dir)
    sherlock_user_properties = yaml.load(open(file_dir), yaml.SafeLoader)
    user_properties = yaml.load(open(args.properties), yaml.SafeLoader)
    sherlock_user_properties.update(user_properties)

    # Build ObjectInfos from properties
    object_infos = []
    mission_object_infos = []
    ffi_object_infos = []
    ffi_coords_object_infos = []
    input_object_infos = []
    input_id_object_infos = []

    ## Adding by-sector analysis objects
    if sherlock_user_properties["SECTOR_TWO_MIN_IDS"]:
        for two_min_id, sectors in sherlock_user_properties["SECTOR_TWO_MIN_IDS"].items():
            star_info = get_star_info(sherlock_user_properties, two_min_id)
            aperture = get_aperture(sherlock_user_properties, two_min_id)
            if sectors == 'all':
                mission_object_infos.append(MissionObjectInfo(two_min_id, sectors, star_info=star_info, aperture_file=aperture))
            else:
                for sector in sectors:
                    mission_object_infos.append(MissionObjectInfo(two_min_id, sector, star_info=star_info, aperture_file=aperture))
    if sherlock_user_properties["SECTOR_FFI_IDS"]:
        for ffi_id, sectors in sherlock_user_properties["SECTOR_FFI_IDS"].items():
            star_info = get_star_info(sherlock_user_properties, None)
            aperture = get_aperture(sherlock_user_properties, ffi_id)
            if sectors == 'all':
                ffi_object_infos.append(MissionFfiIdObjectInfo(ffi_id, sectors, star_info=star_info, aperture_file=aperture))
            else:
                for sector in sectors:
                    ffi_object_infos.append(MissionFfiIdObjectInfo(ffi_id, sector, star_info=star_info, aperture_file=aperture))
    if sherlock_user_properties["SECTOR_FFI_COORDINATES"]:
        for coords, sectors in sherlock_user_properties["SECTOR_FFI_COORDINATES"].items():
            star_info = get_star_info(sherlock_user_properties, str(coords[0]) + "_" + str(coords[1]))
            aperture = get_aperture(sherlock_user_properties, coords)
            if sectors == 'all':
                ffi_object_infos.append(MissionFfiCoordsObjectInfo(coords[0], coords[1], sectors, star_info=star_info, aperture_file=aperture))
            else:
                for sector in sectors:
                    ffi_coords_object_infos.append(MissionFfiCoordsObjectInfo(coords[0], coords[1], sector, star_info=star_info, aperture_file=aperture))

    ## Adding global analysis objects
    if sherlock_user_properties["GLOBAL_TWO_MIN_IDS"]:
        [mission_object_infos.append(MissionObjectInfo(two_min_id, sectors,
                                                       star_info=get_star_info(sherlock_user_properties, two_min_id),
                                                       aperture_file=get_aperture(sherlock_user_properties, two_min_id)))
         for two_min_id, sectors in sherlock_user_properties["GLOBAL_TWO_MIN_IDS"].items()]
    if sherlock_user_properties["GLOBAL_FFI_IDS"]:
        [ffi_object_infos.append(MissionFfiIdObjectInfo(ffi_id, sectors,
                                                        star_info=get_star_info(sherlock_user_properties, ffi_id),
                                                       aperture_file=get_aperture(sherlock_user_properties, ffi_id)))
         for ffi_id, sectors in sherlock_user_properties["GLOBAL_FFI_IDS"].items()]
    if sherlock_user_properties["GLOBAL_FFI_COORDINATES"]:
        [ffi_coords_object_infos.append(MissionFfiCoordsObjectInfo(coords[0], coords[1], sectors,
                                                       star_info=get_star_info(sherlock_user_properties, str(coords[0]) + "_" + str(coords[1])),
                                                       aperture_file=get_aperture(sherlock_user_properties, coords)))
         for coords, sectors in sherlock_user_properties["GLOBAL_FFI_COORDINATES"].items()]
    if sherlock_user_properties["INPUT_FILES_WITH_IDS"]:
        [input_object_infos.append(
            MissionInputObjectInfo(input_id, sherlock_user_properties["INPUT_FILES_WITH_IDS"][input_id],
                                   star_info=get_star_info(sherlock_user_properties, input_id)))
         for input_id in sherlock_user_properties["INPUT_FILES_WITH_IDS"].keys()]
    if sherlock_user_properties["INPUT_FILES"]:
        [input_id_object_infos.append(InputObjectInfo(file, star_info=get_star_info(sherlock_user_properties, file)))
         for file in sherlock_user_properties["INPUT_FILES"]]

    ## Set mask to object infos
    if sherlock_user_properties["TWO_MIN_MASKS"]:
        for object_info in mission_object_infos:
            if object_info.mission_id() in sherlock_user_properties["TWO_MIN_MASKS"].keys():
                object_info.initial_mask = sherlock_user_properties["TWO_MIN_MASKS"][object_info.mission_id()]
    if sherlock_user_properties["FFI_IDS_MASKS"]:
        for object_info in ffi_object_infos:
            if object_info.mission_id() in sherlock_user_properties["FFI_IDS_MASKS"].keys():
                object_info.initial_mask = sherlock_user_properties["FFI_IDS_MASKS"][object_info.mission_id()]
    if sherlock_user_properties["FFI_COORDINATES_MASKS"]:
        for object_info in ffi_coords_object_infos:
            key = str(object_info.ra) + "_" + str(object_info.dec)
            if key in sherlock_user_properties["FFI_COORDINATES_MASKS"].keys():
                object_info.initial_mask = sherlock_user_properties["FFI_COORDINATES_MASKS"][key]
    if sherlock_user_properties["INPUT_FILES_MASKS"]:
        for object_info in input_object_infos:
            if object_info.input_file in sherlock_user_properties["INPUT_FILES_MASKS"].keys():
                object_info.initial_mask = sherlock_user_properties["INPUT_FILES_MASKS"][object_info.input_file]
    if sherlock_user_properties["INPUT_FILES_WITH_IDS_MASKS"]:
        for object_info in input_id_object_infos:
            if object_info.mission_id() in sherlock_user_properties["INPUT_FILES_WITH_IDS_MASKS"].keys():
                object_info.initial_mask = sherlock_user_properties["INPUT_FILES_WITH_IDS_MASKS"][object_info.mission_id()]

    ## Set detrend period to object infos
    if sherlock_user_properties["TWO_MIN_INITIAL_DETREND_PERIOD"]:
        for object_info in mission_object_infos:
            if object_info.mission_id() in sherlock_user_properties["TWO_MIN_INITIAL_DETREND_PERIOD"].keys():
                object_info.initial_detrend_period = sherlock_user_properties["TWO_MIN_INITIAL_DETREND_PERIOD"][object_info.mission_id()]
    if sherlock_user_properties["FFI_IDS_INITIAL_DETREND_PERIOD"]:
        for object_info in ffi_object_infos:
            if object_info.mission_id() in sherlock_user_properties["FFI_IDS_INITIAL_DETREND_PERIOD"].keys():
                object_info.initial_detrend_period = sherlock_user_properties["FFI_IDS_INITIAL_DETREND_PERIOD"][object_info.mission_id()]
    if sherlock_user_properties["FFI_COORDINATES_INITIAL_DETREND_PERIOD"]:
        for object_info in ffi_coords_object_infos:
            key = str(object_info.ra) + "_" + str(object_info.dec)
            if key in sherlock_user_properties["FFI_COORDINATES_INITIAL_DETREND_PERIOD"].keys():
                object_info.initial_detrend_period = sherlock_user_properties["FFI_COORDINATES_INITIAL_DETREND_PERIOD"][key]
    if sherlock_user_properties["INPUT_FILES_INITIAL_DETREND_PERIOD"]:
        for object_info in input_object_infos:
            if object_info.input_file in sherlock_user_properties["INPUT_FILES_INITIAL_DETREND_PERIOD"].keys():
                object_info.initial_detrend_period = sherlock_user_properties["INPUT_FILES_INITIAL_DETREND_PERIOD"][
                    object_info.input_file]
    if sherlock_user_properties["INPUT_FILES_WITH_IDS_INITIAL_DETREND_PERIOD"]:
        for object_info in input_id_object_infos:
            if object_info.mission_id() in sherlock_user_properties["INPUT_FILES_WITH_IDS_INITIAL_DETREND_PERIOD"].keys():
                object_info.initial_detrend_period = \
                sherlock_user_properties["INPUT_FILES_WITH_IDS_INITIAL_DETREND_PERIOD"][object_info.mission_id()]
    user_search_zone = None
    user_prepare = None
    user_selection_algorithm = None
    from pathlib import Path
    if sherlock_user_properties["CUSTOM_SEARCH_ZONE"] is not None:
        user_search_zone = load_module(sherlock_user_properties["CUSTOM_SEARCH_ZONE"])
        class_name = Path(sherlock_user_properties["CUSTOM_SEARCH_ZONE"].replace(".py", "")).name
        user_search_zone = getattr(user_search_zone, class_name)
        globals()[class_name] = user_search_zone
        pickle.dumps(user_search_zone)
        user_search_zone = user_search_zone()
    if sherlock_user_properties["PREPARE_ALGORITHM"] is not None:
        user_prepare = load_module(sherlock_user_properties["PREPARE_ALGORITHM"])
        class_name = Path(sherlock_user_properties["PREPARE_ALGORITHM"].replace(".py", "")).name
        user_prepare = getattr(user_prepare, class_name)()
        globals()[class_name] = user_prepare
        pickle.dumps(user_prepare)
    if sherlock_user_properties["CUSTOM_SELECTION_ALGORITHM"] is not None:
        user_selection_algorithm = load_module(sherlock_user_properties["CUSTOM_SELECTION_ALGORITHM"])
        class_name = Path(sherlock_user_properties["CUSTOM_SELECTION_ALGORITHM"].replace(".py", "")).name
        user_selection_algorithm = getattr(user_selection_algorithm, class_name)()
        globals()[class_name] = user_selection_algorithm
        pickle.dumps(user_selection_algorithm)
    ## Adding all object infos to same array
    object_infos.extend(mission_object_infos)
    object_infos.extend(ffi_object_infos)
    object_infos.extend(ffi_coords_object_infos)
    object_infos.extend(input_object_infos)
    object_infos.extend(input_id_object_infos)
    sherlock.Sherlock(sherlock_user_properties["UPDATE_OIS"], object_infos, args.explore) \
        .setup_detrend(sherlock_user_properties["INITIAL_SMOOTH_ENABLED"],
                       sherlock_user_properties["INITIAL_HIGH_RMS_MASK"],
                       sherlock_user_properties["INITIAL_HIGH_RMS_THRESHOLD"],
                       sherlock_user_properties["INITIAL_HIGH_RMS_BIN_HOURS"],
                       sherlock_user_properties["DETRENDS_NUMBER"],
                       sherlock_user_properties["DETREND_METHOD"], sherlock_user_properties["DETREND_L_MIN"],
                       sherlock_user_properties["DETREND_L_MAX"], sherlock_user_properties["DETREND_CORES"],
                       sherlock_user_properties["AUTO_DETREND_ENABLED"], sherlock_user_properties["AUTO_DETREND_RATIO"],
                       sherlock_user_properties["AUTO_DETREND_METHOD"],
                       user_prepare) \
        .setup_transit_adjust_params(sherlock_user_properties["MAX_RUNS"], sherlock_user_properties["MIN_SECTORS"],
                                     sherlock_user_properties["MAX_SECTORS"],
                                     sherlock_user_properties["PERIOD_PROTECT"],
                                     sherlock_user_properties["SEARCH_ZONE"], user_search_zone,
                                     sherlock_user_properties["PERIOD_MIN"], sherlock_user_properties["PERIOD_MAX"],
                                     sherlock_user_properties["BIN_MINUTES"], sherlock_user_properties["CPU_CORES"],
                                     sherlock_user_properties["SNR_MIN"], sherlock_user_properties["SDE_MIN"],
                                     sherlock_user_properties["FAP_MAX"], sherlock_user_properties["MASK_MODE"],
                                     sherlock_user_properties["BEST_SIGNAL_ALGORITHM"],
                                     sherlock_user_properties["QUORUM_STRENGTH"],
                                     sherlock_user_properties["MIN_QUORUM"],
                                     user_selection_algorithm,
                                     sherlock_user_properties["FIT_METHOD"],
                                     sherlock_user_properties["OVERSAMPLING"],
                                     sherlock_user_properties["T0_FIT_MARGIN"],
                                     sherlock_user_properties["DURATION_GRID_STEP"]) \
        .run()



from abc import ABC, abstractmethod


class StarCatalog(ABC):
    sun_teff = 5700

    def __init__(self):
        pass

    @abstractmethod
    def catalog_info(self, id):
        pass

    def star_luminosity(self, t_eff, star_radius):
        return (star_radius ** 2) * (t_eff ** 4) / (self.sun_teff ** 4)



import logging
import lightkurve as lk
from sherlockpipe.star import starinfo
from sherlockpipe.objectinfo.MissionInputObjectInfo import MissionInputObjectInfo
from sherlockpipe.objectinfo.preparer.LightcurveBuilder import LightcurveBuilder
import pandas as pd


class MissionInputLightcurveBuilder(LightcurveBuilder):
    def __init__(self):
        super().__init__()

    def build(self, object_info, sherlock_dir):
        mission_id = object_info.mission_id()
        sherlock_id = object_info.sherlock_id()
        quarters = None
        sectors = None
        if isinstance(object_info, MissionInputObjectInfo):
            logging.info("Retrieving star catalog info...")
            mission, mission_prefix, id = super().parse_object_id(mission_id)
            if mission_prefix not in self.star_catalogs:
                raise ValueError("Wrong object id " + mission_id)
            star_info = starinfo.StarInfo(sherlock_id, *self.star_catalogs[mission_prefix].catalog_info(id))
        else:
            star_info = starinfo.StarInfo(sherlock_id)
            star_info.assume_model_mass()
            star_info.assume_model_radius()
        logging.info("Loading lightcurve from file " + object_info.input_file + ".")
        df = pd.read_csv(object_info.input_file, float_precision='round_trip', sep=',',
                         usecols=['#time', 'flux', 'flux_err'])
        lc = lk.LightCurve(time=df['#time'], flux=df['flux'], flux_err=df['flux_err'])
        transits_min_count = 1
        return lc, star_info, transits_min_count, sectors, quarters



import os
import ast
import csv
import sys
import warnings
import matplotlib
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from argparse import ArgumentParser

#froms from standards
from tkinter import simpledialog
from matplotlib.ticker import AutoMinorLocator
from matplotlib.ticker import FormatStrFormatter

#froms from non-standards
from os.path import exists

#custom modules to import
from LATTE import LATTEutils as utils
warnings.filterwarnings('ignore')

'''
Overview of LATTE scipts:

__main__.py      : Intitialises the parameters, what TIC ID, sector, checks for downloads of the data, FFI or not? 

LATTEutils.py    : All the functions needed to download data and text files, runs the interactive gui, all of the plotting and data handling. 

LATTE_DV.py      : Scipt to combine all of the results from LATTEbrew in order to generate a pdf data validation report.

LATTEbrew.py     : Calls the LATTEutils functions in turn in order to store the results and keeps track of what has been generated. Calls the LATTE_DV.py function to collate all the results.


This scipt has two functions, one for the short cadence data and one for the FFI data. 
These functions are either called from within __main__.py (if run with input target list), or from within the interact functions in LATTEutils.py (normal mode).

Brew keeps track if any part of the code crashes (e.g. the downloading of the TPF - so that these are omitted in the generation of the DV report).
'''

def brew_LATTE(tic, indir, syspath, transit_list, simple, BLS, model, save, DV, sectors, sectors_all, alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltime12, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad, ra, dec, args):
	'''
	This function combines all the results from LATTE and calls all the different functions - 
	it makes the plots, saves them, runs the BLS model and the pyaneti model before making a PHT DV report (if this option is selected.) 
	
	Parameters
	----------
	tic  :   str
		target TIC ID
	indir  :  str
		path to directory where all the plots and data will be saved. 
	transit_list   : list
		list of the transit-like events
	simple   :   boolean
		whether or not to run the simple version
	BLS   :   boolean
		whether or not to run the BLS routine
	model   :   boolean
		whether or not to model the transit using pyaneti
	save   :	boolean
		whether or not to save the figures and data
	DV   :   boolean
		whether or not to write and save a DV report
	sectors_all  :   list
		all the sectors in which the target has been/ will be observed
	alltime  :  list
		times (not binned)
	allflux  :  list
		normalized flux (not binned)
	allflux_err  :  list
		normalized flux errors (not binned)
	all_md  :  list
		times of the momentum dumps
	alltimebinned  :  list
		binned time
	allfluxbinned  :  list
		normalized binned flux
	allx1  :  list
		CCD column position of target’s flux-weighted centroid. In x direction
	allx2  :  list
		The CCD column local motion differential velocity aberration (DVA), pointing drift, and thermal effects. In x direction
	ally1  :  list
		CCD column position of target’s flux-weighted centroid. In y direction
	ally2  :  list
		The CCD column local motion differential velocity aberration (DVA), pointing drift, and thermal effects. In y direction
	alltimel2  :  list
		time used for the x and y centroid position plottin
	allfbkg  :  list
		background flux
	start_sec  :  list
		times of the start of the sector
	end_sec  :  list
		times of the end of the sector
	in_sec  :  list
		the sectors for which data was downloaded
	tessmag  :  list
		TESS magnitude of the target star
	teff  :  float
		effective temperature of the tagret star (K)
	srad  :  float
		radius of the target star (solar radii)
	 ra	:	float 
		the right ascension of the target stars
	 dec	:   float 
		the declination of the target star

	'''

	# -------------------
	# SAVE THE DATA FILES 
	# -------------------

	if (save == True) or (DV == True):
		save = True
		# if this folder doesn't exist then create it. These are the folder where the images, data and reports for each TIC ID will be stored.
		newpath = '{}/{}'.format(indir,tic)
		
		if not exists(newpath):
			os.makedirs(newpath)

		# save the data used as a text file - these often come in use later for a quick re-analysis.
		with open('{}/{}/{}_data.txt'.format(indir, tic, tic), "w") as f:

			# get rid of nan values first using a mask
			good_mask = np.isfinite(np.array(alltime)) * np.isfinite(np.array(allflux)) * np.isfinite(np.array(allflux_err)) 
			alltime_ar = np.array(alltime)[good_mask]
			allflux_ar = np.array(allflux)[good_mask]
			allflux_err_ar = np.array(allflux_err)[good_mask]

			# save
			writer = csv.writer(f, delimiter='\t')
			writer.writerow(['time', 'flux', 'flux_err'])
			writer.writerows(zip(alltime_ar,allflux_ar,allflux_err_ar))
		
		'''
		if the modelling option was also chose, save another data file with slightly different formatting to be called by Pyaneti.
		Pyaneti requires a very specific data format.
		
		Furhermore, in order for Pyaneti to run more efficiently (it has a Bayesian backend which scales with number of data points)
		we create a cutout of the times around the times of the marked transit events. 
		'''

		if model == True:
			with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "w") as f:
				writer = csv.writer(f, delimiter='\t')
				writer.writerow(['#time', 'flux', 'flux_err'])

			# If the dip separations are too small, then don't create cut outs and save the whole dataset
			if (len(transit_list) > 1) and ((transit_list[1] - transit_list[0]) < 2): # if there are LOTS of transit events on short period (if so it's probably a TOI but let's keep it here as a condition)
				with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "a") as f:
					writer = csv.writer(f, delimiter='\t')
					writer.writerows(zip(alltime_ar,allflux_ar,allflux_err_ar)) # save all the data

			# else create a cut out of the data around the time of the transit events
			else:
				for transit in transit_list:
					# save the data 
					# get rid of nan values first - this is used for the pyaneti code
					pyaneti_mask = (alltime_ar > (transit - 1)) * (alltime_ar < (transit + 1))

					with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "a") as f:
						writer = csv.writer(f, delimiter='\t')
						writer.writerows(zip(alltime_ar[pyaneti_mask],allflux_ar[pyaneti_mask],allflux_err_ar[pyaneti_mask]))


	# -----------------------------------
	#			START PLOTTING		  	 - calls functions from LATTEutils.py
	# -----------------------------------
	
	# create a plot of the fulllighcurves with the momentum dumps (MDs) marked and a zoom-in of the marked transits
	# this plit is saved but not shown (as already shown in the interact part fo the code)
	utils.plot_full_md(tic, indir, alltime,allflux,all_md,alltimebinned,allfluxbinned, transit_list, args)

	# Get a list of the sectors that have transit marked in them
	# this is so that we no longer have to loop through all of the sectors, and can focus on the ones which are important.
	transit_sec = utils.transit_sec(in_sec, start_sec, end_sec, transit_list)
	
	# -----------

	# plot how the centroids moved during the transit event
	utils.plot_centroid(tic, indir,alltime12, allx1, ally1, allx2, ally2, transit_list, args)
	# plot the background flux at the time of the transit event.
	utils.plot_background(tic, indir,alltime, allfbkg, transit_list, args)
	
	print ("Centroid and background plots... done.")
	# -----------

	# if the 'simple' option is chosen in the GUI, then the code will end here - this is designed to provide a quick analysis requiring no TPFs.
	if simple == True:
		print ("Simple option was selected, therefore end analysis here.")
		sys.exit('')

	# -----------

	# call function to extract the Target Pixel File information 
	# this is needed in order to extract the LCs in different aperture sizes.
	# the data is extracted using the open source Lightkurve package as they a built in function to extract LCs using different aperture sizes
	TESS_unbinned_t_l, TESS_binned_t_l, small_binned_t_l, TESS_unbinned_l, TESS_binned_l, small_binned_l, tpf_list = utils.download_tpf_lightkurve(indir, transit_list, sectors, tic)
	
	# if the TPF wasn't corrupt then make the TPF files (only very ocassionally corrupt but don't want code to crash if it is corrrupt)
	if (TESS_unbinned_t_l != -111):

		tpf_corrupt = False
		# plot the LCs using two different aperture sizes.
		utils.plot_aperturesize(tic,indir,TESS_unbinned_t_l, TESS_binned_t_l, small_binned_t_l, TESS_unbinned_l, TESS_binned_l, small_binned_l, transit_list, args)
		
		print ("Aperture size plots... done.")
		# ------------
	
		'''
		Plot the average pixel brightness of the cut-out around the target star and the corresponding SDSS field of view.
		Both are oriented so that North is pointing upwards.
		The former also shows the nearby stars with TESS magnitude brighter than 17. Queried from GAIA using astroquery.
		The function returns the mass of the star (also output from astroquery)- this is a useful input for the Pyaneti modelling		
		'''
		if args.mpi == False:
			test_astroquery, _, _, mstar = utils.plot_TESS_stars(tic,indir, transit_list, transit_sec, tpf_list, args)
		else:
			test_astroquery, _, _, mstar = utils.plot_TESS_stars_not_proj(tic,indir, transit_list, transit_sec, tpf_list, args)

		# keep track of whether astroquery is working (sometimes the site is down and we don't want this to stop us from running the code)
		astroquery_corrupt = False

		if test_astroquery == -999:
			astroquery_corrupt = True
			print ("Star Aperture plots... failed.")
		
		else:
			print ("Star Aperture plots... done.")
		
		# ------------
		
		# Download the Target Pixel File using the raw MAST data - this comes in a different format as the TPFs extracted using Lightkurve
		# This data is then corrected using Principal Component Analysis is orderto get rid of systematics.
		X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list, t_list, T0_list, tpf_filt_list = utils.download_tpf_mast(indir, transit_sec, transit_list, tic)
		
		# ------------
		
		'''
		plot the in and out of transit flux comparison.
		By default the images are NOT oriented north - this is because the reprojection takes longer to run and for a simple
		analysis to check whether the brightest pixel moves during the transit this is not required.
		The orientation towards north can be defined in the command line with '--north'.
		'''
		if args.north == True:
			utils.plot_in_out_TPF_proj(tic, indir, X4_list, oot_list, t_list, intr_list, T0_list, tpf_filt_list, tpf_list, args)
			print ("In and out of aperture flux comparison with reprojection... done. ")
		
		else:
			utils.plot_in_out_TPF(tic, indir, X4_list, oot_list, t_list, intr_list, T0_list, tpf_filt_list, args)
			print ("In and out of aperture flux comparison... done.")
		# ------------
		
		# For each pixel in the TPF, extract and plot a lightcurve around the time of the marked transit event.
		utils.plot_pixel_level_LC(tic, indir, X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list,t_list, T0_list, args)
		print ("Pixel level LCs plot... done.")
		# ------------
		
	else:
		tpf_corrupt = True
		mstar = 1 # need to define mstar otherwise pyaneti will complain - just make it one as an approximation.
		astroquery_corrupt = True
	
	# ------------
	# end of plots that require target pixel files
	# ------------

	# If more than one transit has been marked by the user, the LC is phase folded based on the period of the separation of the first two maarked peaks.
	# These plots are saved but do not feature in the DV report.
	if len (transit_list) > 1: # needs to know a period so can only do this if more than one transit has been marked.

		period = transit_list[1] - transit_list[0]
		t0 = transit_list[0] # time of the first marking
		
		# calculate the phase
		phased = np.array([-0.5+( ( t - t0-0.5*period) % period) / period for t in alltimebinned])

		fig, ax = plt.subplots(figsize=(5.55,5))
		ax.plot(phased, allfluxbinned, marker='.',color = 'k', alpha = 1, lw = 0, markersize = 4, label = 'None', markerfacecolor='k')
	
		#ax.plot(phased, allflux,marker='o',color = 'navy', alpha = 0.7, lw = 0, markersize = 2, label = 'binning = 7', markerfacecolor='white')
		plt.title("Phase folded LC")
		ax.set_xlabel("Phase (days)")
		ax.set_ylabel("Normalized Flux")
		plt.plot()


		if save == True:
			plt.savefig('{}/{}/{}_phase_folded.png'.format(indir, tic, tic), format='png')
		
		if args.noshow == False:
			plt.show()

		print ("Phase folded plot... done.")

	else:
		print ("\n Only one transit marked - therefore can't be phase folded. \n")
	# ------------

	'''
	Plot LCs of the six closest TESS target stars. This allows us to check whether the transit-like events 
	also appear in other nearby LCs which would be a sign that this is caused by a background event.
	'''

	# get the tic IDs of the six nearest stars
	ticids, distance, target_ra, target_dec = utils.nn_ticids(indir, transit_sec, tic)

	# download the data for these stars
	alltime_nn, allflux_nn, all_md_nn, alltimebinned_nn, allfluxbinned_nn,outtics,tessmag_list, distance = utils.download_data_neighbours(indir, transit_sec[0], ticids, distance)
	
	# plot the LCs
	utils.plot_nn(tic, indir,alltime_nn, allflux_nn, alltimebinned_nn, allfluxbinned_nn, transit_list, outtics, tessmag_list, distance, args)
	print ("Nearest neighbour plot... done.")
	# ------------

	# if the BLS option is chose, a BLS search is run. The LCs are first detrended and smoothed using a moving average. 
	# The corrected and uncorrected LCs are saves as a single plot for comparison and to verify that the correction worked well - saved but do not feature in the DV report. 
	if BLS == True:
		print ("Running BLS algorithm...", end =" ")
		bls_stats1, bls_stats2 = utils.data_bls(tic, indir, alltime, allflux, allfluxbinned, alltimebinned, args)
		print ("done.")
	# ------------

	# SKIP FROM HERE....

	'''
	NOTE: CURRENTLY ONLY WORKS ON NORA'S COMPUTER - WILL BE AVAILABLE IN NEXT RELEASE SO PLEASE SCIP THIS PART OF THE CODE
	If the modelling option is selected (in the GUI), model the transit event using Pyaneti (Barragan et al 2018)
	which uses an Bayesian approach with an MCMC sampling to best fit and model the transit.
	The code runs slightly differently depending on whether one or multiple transits have been marked. 
	This is because with multiple transits the code has information about the possible orbital period.
	Need to ensure that the code has compiled correctly on the users computer. 
	'''

	# First check if Pyaneti is installed...
	if os.path.exists("{}/pyaneti_LATTE.py".format(syspath)):

		if model == True:
			print ("Running Pyaneti modelling - this could take a while so be patient...")
	
			transit_list_model =  ("{}".format(str(np.asarray(transit_list)))[1:-1]) # change the list into a string and get rid of the brackets
			# the code is usually run through the command line so call it using the os.system function.
			
			os.system("python3 {}/pyaneti_LATTE.py {} {} {} {} {} {} {}".format(syspath, tic, indir, syspath, mstar, teff, srad, transit_list_model))
	
	else:
		print ("Pyaneti has not been installed so you can't model anything yet. Contact Nora or Oscar for the LATTE version of the Pyaneti code.")
		model = False

	# ... UNTIL HERE
	# ------------


	# Finally, create a DV report which summarises all of the plots and tables.
	if DV == True: 
		from LATTE import LATTE_DV as ldv

		if BLS == True:
			ldv.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, target_ra, target_dec, tessmag, teff, srad, bls_stats1, bls_stats2, tpf_corrupt, astroquery_corrupt, FFI = False,  bls = True, model = model, mpi = args.mpi)
		else:
			ldv.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, target_ra, target_dec, tessmag, teff, srad, [0], [0], tpf_corrupt, astroquery_corrupt, FFI = False,  bls = False, model = model, mpi = args.mpi)


def brew_LATTE_FFI(tic, indir, syspath, transit_list, simple, BLS, model, save, DV, sectors, sectors_all, alltime, allflux_normal, allflux_small, allflux, all_md, allfbkg, allfbkg_t, start_sec, end_sec, in_sec, X1_list, X4_list, apmask_list, arrshape_list, tpf_filt_list, t_list, bkg_list, tpf_list, ra, dec, args):
	
	'''
	This function that runs LATTE - makes the plots, saves them, runs the BLS model and the pyaneti model before 
	making a PHT DV report (if told to do so.)
	This function is very similar to brew_LATTE - 
	except that it is designed to analyse the FFIs and therefore the data download is different.
	
	Parameters
	----------
	tic  :   str
		target TIC ID
	indir  :  str
		path to directory where all the plots and data will be saved. 
	transit_list   : list
		list of the transit-like events
	simple   :   boolean
		whether or not to run the simple version
	BLS   :   boolean
		whether or not to run the BLS routine
	model   :   boolean
		whether or not to model the transit using pyaneti
	save   :	boolean
		whether or not to save the figures and data
	DV   :   boolean
		whether or not to write and save a DV report
	sectors_all  :   list
		all the sectors in which the target has been/ will be observed

	alltime  :  list
		times
	allflux_normal  :  list
		normalized flux extracted with the larger aperture (PCA corrected)
	allflux_small  : list
		normalized flux extracted with the smaller aperture (PCA corrected)
	allflux   : list 
		normalized detrended flux extracted with the larger aperture
	all_md  :  list
		times of the momentum dumps
	allfbkg  :  list
		background flux
	allfbkg_t  :  list
		times used to plot the background
	start_sec  :  list
		times of the start of the sector
	end_sec  :  list
		times of the end of the sector
	in_sec  :  list
		the sectors for which data was downloaded
	X1_list  :  list
		flux vs time for each pixel (for each sector)
	X4_list  :  list
		PCA corrected flux vs time for each pixel (for each sector)
	apmask_list  :  list
		aperture masks from the pipeline
	arrshape_list  :  list
		list of the shape of the array (for each sector)
	tpf_filt_list   : list
		list of the filtered (masked) corrected target pixel data - from X4. (for each sector)
	t_list  :  list
		list of the time arrays (for each sector)
	bkg_list  :  list
		the flux that was used to normalise each pixel - i.e. what is used to make the background plot colour for each pixel.
	tpf_list   : list 
		list of the target pixel files (for each sector)
	ra   :   float
		right ascension of the target star
	dec   :   float
		declination of the target star

	'''

	# ----------------------------------------
	#		   SAVE THE DATA FILES 
	# ----------------------------------------


	if (save == True) or (DV == True):
		save = True

		# if this folder doesn't exist then create it. These are the folder where the images, data and reports for each TIC ID will be stored.
		newpath = '{}/{}'.format(indir,tic)
		
		if not exists(newpath):
			os.makedirs(newpath)
		
		# save the data used as a text file - these often come in use later for a quick re-analysis.
		with open('{}/{}/{}_data.txt'.format(indir, tic, tic), "w") as f:
			
			# get rid of nan values first using a mask
			good_mask = np.isfinite(np.array(alltime)) * np.isfinite(np.array(allflux))
			alltime_ar = np.array(alltime)[good_mask]
			allflux_ar = np.array(allflux)[good_mask]
			allflux_err_ar = allflux_ar * 0.001

			# save data 
			writer = csv.writer(f, delimiter='\t')
			writer.writerow(['time', 'flux', 'flux_err'])
			writer.writerows(zip(alltime_ar,allflux_ar,allflux_err_ar))

		'''
		if the modelling option was also chose, save another data file with slightly different formatting to be called by Pyaneti.
		Pyaneti requires a very specific data format.
		
		Furhermore, in order for Pyaneti to run more efficiently (it has a Bayesian backend which scales with number of data points)
		we create a cutout of the times around the times of the marked transit events. 
		'''

		if model == True:
			with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "w") as f:
				writer = csv.writer(f, delimiter='\t')
				writer.writerow(['#time', 'flux', 'flux_err'])

			# if the dip separations are too small, then don't create cut outs and save the whole dataset
			if (len(transit_list) > 1) and ((transit_list[1] - transit_list[0]) < 2): # if there are LOTS of transit events on short period (if so it's probably a TOI but let's keep it here as a condition)
				with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "a") as f:
					writer = csv.writer(f, delimiter='\t')
					writer.writerows(zip(alltime_ar,allflux_ar,allflux_err_ar)) # save all the data

			# else create a cut out of the data around the time of the transit event
			else:
				for transit in transit_list:
					# save the data 
					# get rid of nan values first - this is used for the pyaneti code
					pyaneti_mask = (alltime_ar > (transit - 1)) * (alltime_ar < (transit + 1))

					with open('{}/{}/{}_data_pyaneti.dat'.format(indir, tic, tic), "a") as f:
						writer = csv.writer(f, delimiter='\t')
						writer.writerows(zip(alltime_ar[pyaneti_mask],allflux_ar[pyaneti_mask],allflux_err_ar[pyaneti_mask]))


	# ------------------------------------------
	#			   START PLOTTING
	# ------------------------------------------
	
	# create a plot of the fulllighcurves with the momentum dumps (MDs) marked and a zoom-in of the marked transits
	utils.plot_full_md(tic, indir, alltime, allflux, all_md, alltime, allflux, transit_list, args)

	# get the sectors that have a transit marked in them
	transit_sec = utils.transit_sec(in_sec,start_sec, end_sec, transit_list)
	
	# -----------

	# plot the background flux at the time of the transit event.
	utils.plot_background(tic, indir, allfbkg_t, allfbkg, transit_list, args)
	print ("Background plots... done.")
	# -----------

	if simple == True:
		print ("Simple option was selected, therefore end analysis here.")
		sys.exit('')
	# -----------

	# plot the LC using different aperture sizes
	utils.plot_aperturesize(tic,indir,alltime, alltime, alltime, allflux_normal, allflux_normal, allflux_small, transit_list, args)
	
	print ("Aperture size plots... done.")
	# -----------

	'''
	Plot the average pixel brightness of the cut-out around the target star and the corresponding SDSS field of view.
	Both are oriented so that North is pointing upwards.
	The former also shows the nearby stars with TESS magnitude brighter than 17. Queried from GAIA using astroquery.
	The function returns the TESS magnitude, effective temperature (K), the radius of the star (solar radii)and the 
	mass of the star (solar mass) (also output from astroquery)- this is a useful input for the Pyaneti modelling		
	'''

	if args.mpi == False:
		tessmag, teff, srad, mstar = utils.plot_TESS_stars(tic,indir,transit_list, transit_sec, tpf_list, args)
	else:
		tessmag, teff, srad, mstar = utils.plot_TESS_stars_not_proj(tic,indir,transit_list, transit_sec, tpf_list, args)

	# keep track of whether astroquery is working (sometimes the site is down and we don't want this to stop us from running the code)
	astroquery_corrupt = False

	if tessmag == -999:
		astroquery_corrupt = True
		print ("Star Aperture plots... failed.")
	
	else:
		print ("Star Aperture plots... done.")
		

	# -----------

	# If more than one transit has been marked by the user, the LC is phase folded based on the period of the separation of the first two maarked peaks.
	# These plots are saved but do not feature in the DV report.

  # needs to know a period so can only do this if more than one transit has been marked.
	if len(transit_list) > 1:
		# find the period
		period = transit_list[1] - transit_list[0]
		t0 = transit_list[0] # time of the first marking
		
		# calculate the phase
		#phased = np.array([-0.5+( ( t - t0-0.5*period) % period) / period for t in alltime])

		# phased plot where odd and even are different colours
		phased = (np.array(alltime) - t0) % period
		phased2 = (np.array(alltime) - (t0)) % (period*2)
		
		index = phased>0.5*period
		index2 = phased2 >0.5*(period *2)
		
		phased[index] -= period
		phased2[index2] -= (period *2)
	
		fig, ax = plt.subplots(figsize=(5.55,5))

		ax.plot(phased,np.array(allflux), 'k.', markersize=5)
		ax.plot(phased2,np.array(allflux), 'r.', markersize=5)
	
		#ax.plot(phased, allflux,marker='o',color = 'navy', alpha = 0.7, lw = 0, markersize = 2, label = 'binning = 7', markerfacecolor='white')
		plt.title("Phase folded LC")
		ax.set_xlabel("Phase (days)")
		ax.set_ylabel("Normalized Flux")
		plt.plot()

		if save == True:
			plt.savefig('{}/{}/{}_phase_folded.png'.format(indir, tic, tic), format='png')
		
		if args.noshow == False:
			plt.show()

		print ("Phase folded plot... done.")

	else:
		print ("\n Only one transit marked - therefore can't be phase folded. \n")	
	# ------------

	'''
	In order to create the plots that compare the in and out of transit average flux 
	we need to create TPF masks for the in and out of transit. 
	This couldn't be done at the time of the extraction of the TPF arrays as the transit times 
	had not yet been defined. 
	'''

	oot_list = [] # out of transit
	intr_list = []  # in transit

	X1_list_n = []
	X4_list_n = []
	bkg_list_n = []
	apmask_list_n = []
	arrshape_list_n = []
	t_list_n = []

	tpf_filt_list_n = []
	tpf_list_n = []

	T0_list = []  # list of the transit times - make a new list to ensure that thet are in the same order.
	

	for T0 in transit_list:
		for idx,t in enumerate(t_list):
			# these times were found to give good approximations for out of transit and in transit
			# Note: this will NOT be 'perfect' for all events, as it depends on the transit duration which the code doesnt' know at this time but it's a good approximation.
			if (T0 > np.nanmin(t)) and (T0 < np.nanmax(t)): # if T0 is within the time... (should be)
				
				oot_list.append((abs(T0-np.array(t)) < 0.55) * (abs(T0-np.array(t)) < 0.3)) 
				intr_list.append(abs(T0-np.array(t)) < 0.1)
				
				X1_list_n.append(X1_list[idx])
				X4_list_n.append(X4_list[idx])
				bkg_list_n.append(bkg_list[idx])
				apmask_list_n.append(apmask_list[idx])
				arrshape_list_n.append(arrshape_list[idx])
				t_list_n.append(t)

				tpf_filt_list_n.append(tpf_filt_list[idx])
				tpf_list_n.append(tpf_list[idx])

				T0_list.append(T0)

	X1_list	 	= 	X1_list_n
	X4_list	 	= 	X4_list_n
	bkg_list		= 	bkg_list_n
	apmask_list 	= 	apmask_list_n
	arrshape_list   =   arrshape_list_n
	t_list 			=   t_list_n
	tpf_filt_list 	= 	tpf_filt_list_n
	tpf_list 		=   tpf_list_n
	
	# ------------


	# Plot the in and out of transit flux comparison 
	# By default the images are NOT orented north - this is because the reprojectio takes longer to run and for a simple
	# analysis to chekc whether the brightest pixel moves during the transit this is not required.
	# The orientation towards north can be defined in the command line with '--north'.

	if args.north == True:
		utils.plot_in_out_TPF_proj(tic, indir, X4_list, oot_list, t_list, intr_list, T0_list, tpf_filt_list, tpf_list, args)
		print ("In and out of aperture flux comparison with reprojection... done. ")
	else:
		utils.plot_in_out_TPF(tic, indir, X4_list, oot_list, t_list, intr_list, T0_list, tpf_filt_list, args)
		print ("In and out of aperture flux comparison... done.")
	# ------------

	# For each pixel in the TPF, extract and plot a lightcurve around the time of the marked transit event.
	utils.plot_pixel_level_LC(tic, indir, X1_list, X4_list, oot_list, intr_list, bkg_list, apmask_list, arrshape_list, t_list, T0_list, args)
	print ("Pixel level LCs plot... done.")
	# ------------

	# If the BLS option is chose, a BLS search is run. The LCs are first detrended and smoothed using a moving average. 
	# The corrected and uncorrected LCs are saves as a single plot for comparison and to verify that the correction worked well - saved but do not feature in the DV report. 
	if BLS == True:
		print ("Running BLS")
		bls_stats1, bls_stats2 = utils.data_bls_FFI(tic, indir, alltime, allflux, args)
	# ------------

	'''
	If the modelling option is selected (in the GUI), model the transit event using Pyaneti (Barragan et al 2018)
	which uses an Bayesian approach with an MCMC sampling to best fit and model the transit.
	The code runs slightly differently depending on whether one or multiple transits have been marked. 
	This is because with multiple transits the code has information about the possible orbital period.
	Need to ensure that the code has compiled correctly on the users computer. 

	SKIP FOR NOW - this will become available in the next version of LATTE
	'''

	# first check if Pyaneti is even installed...

	if os.path.exists("{}/pyaneti_LATTE.py".format(syspath)):

		if model == True:
			print ("Running Pyaneti modelling - this could take a while so be patient...")
			
			# the code is usually run through the command line so call it using the os.system function.
			transit_list_model =  ("{}".format(str(np.asarray(transit_list)))[1:-1]) # change the list into a string and get rid of the brackets
			os.system("python3 {}/pyaneti_LATTE.py {} {} {} {} {} {} {}".format(syspath, tic, indir, syspath, mstar, teff, srad, transit_list_model))
	
	else:
		print ("Pyaneti has not been installed so you can't model anything yet. Ask Nora or Oscar for the LATTE version of the Pyaneti code.")
		model = False
	
	# SKIP until HERE
	
	# ------------
	# Finally, create a DV report which summarises all of the plots and tables.
	if DV == True: 
		from LATTE import LATTE_DV as ldv

		if BLS == True:
			ldv.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, ra, dec, tessmag, teff, srad, bls_stats1, bls_stats2, False, astroquery_corrupt, FFI = True, bls = True, model = model, mpi = args.mpi)
		else:
			ldv.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, ra, dec, tessmag, teff, srad, [0], [0], False, astroquery_corrupt, FFI = True, bls = False, model = model, mpi = args.mpi)

	else:
		print ("\n  Complete! \n ")



import numpy as np

from sherlockpipe.scoring.SignalSelector import SignalSelector, SignalSelection


class BasicSignalSelector(SignalSelector):
    def __init__(self):
        super().__init__()

    def select(self, transit_results, snr_min, detrend_method, wl):
        detrends_snr = np.nan_to_num([transit_result.snr
                                      for key, transit_result in transit_results.items()])
        best_signal_snr = np.nanmax(detrends_snr)
        best_signal_snr_index = np.nanargmax(detrends_snr)
        best_signal = transit_results[best_signal_snr_index]
        if best_signal_snr > snr_min:  # and SDE[a] > SDE_min and FAP[a] < FAP_max):
            best_signal_score = 1
        else:
            best_signal_score = 0
        return SignalSelection(best_signal_score, best_signal_snr_index, best_signal)


from abc import ABC, abstractmethod


class SignalSelector(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def select(self, transit_result_detrends, snr_min, detrend_method, wl):
        pass

class SignalSelection:
    def __init__(self, score, curve_index, transit_result):
        self.score = score
        self.curve_index = curve_index
        self.transit_result = transit_result

    def get_message(self):
        curve_name = "PDCSAP_FLUX" if self.curve_index == 0 else str(self.curve_index - 1)
        return "Chosen signal with BASIC algorithm --> NAME: " + curve_name + \
               "\tPeriod:" + str(self.transit_result.period) + \
               "\tSNR: " + str(self.transit_result.snr) + \
               "\tSDE: " + str(self.transit_result.sde) + \
               "\tFAP: " + str(self.transit_result.fap) + \
               "\tBORDER_SCORE: " + str(self.transit_result.border_score)



import logging
import numpy as np
from sherlockpipe.star import starinfo
from sherlockpipe.objectinfo.ObjectProcessingError import ObjectProcessingError
from sherlockpipe.objectinfo.preparer.LightcurveBuilder import LightcurveBuilder
import lightkurve as lk
import matplotlib.pyplot as plt
import csv


class MissionLightcurveBuilder(LightcurveBuilder):
    def __init__(self):
        super().__init__()

    def build(self, object_info, sherlock_dir):
        mission_id = object_info.mission_id()
        sherlock_id = object_info.sherlock_id()
        quarters = None
        sectors = None
        logging.info("Retrieving star catalog info...")
        mission, mission_prefix, id = super().parse_object_id(mission_id)
        if mission_prefix not in self.star_catalogs:
            raise ValueError("Wrong object id " + mission_id)
        star_info = starinfo.StarInfo(sherlock_id, *self.star_catalogs[mission_prefix].catalog_info(id))
        logging.info("Downloading lightcurve files...")
        if object_info.aperture_file is None:
            if mission == "TESS" and object_info.sectors != 'all':
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short", sector=object_info.sectors)\
                    .download_all()
            elif mission == "TESS":
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short").download_all()
            elif object_info.sectors != 'all':
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short", quarter=object_info.sectors)\
                    .download_all()
            else:
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="short").download_all()
            if lcf is None:
                raise ObjectProcessingError("Light curve not found for object id " + mission_id)
            lc = None
            matching_objects = []
            for i in range(0, len(lcf.PDCSAP_FLUX)):
                if lcf.PDCSAP_FLUX[i].label == mission_id:
                    if lc is None:
                        lc = lcf.PDCSAP_FLUX[i].normalize()
                    else:
                        lc = lc.append(lcf.PDCSAP_FLUX[i].normalize())
                else:
                    matching_objects.append(lcf.PDCSAP_FLUX[i].label)
            if len(matching_objects) > 0:
                logging.warning("================================================")
                logging.warning("TICS IN THE SAME PIXEL: " + str(matching_objects))
                logging.warning("================================================")
            lc = lc.remove_nans()
            transits_min_count = 1 if len(lcf) == 0 else 2
            if mission_prefix == self.MISSION_ID_KEPLER or mission_id == self.MISSION_ID_KEPLER_2:
                quarters = [lcfile.quarter for lcfile in lcf]
            elif mission_prefix == self.MISSION_ID_TESS:
                sectors = [file.sector for file in lcf]
            if mission_prefix == self.MISSION_ID_KEPLER_2:
                logging.info("Correcting K2 motion in light curve...")
                quarters = [lcfile.campaign for lcfile in lcf]
                lc = lc.to_corrector("sff").correct(windows=20)
            return lc, star_info, transits_min_count, np.unique(sectors), np.unique(quarters)
        else:
            logging.info("Using user apertures!")
            if mission == "TESS" and object_info.sectors != 'all':
                tpfs = lk.search_targetpixelfile(str(mission_id), mission=mission, cadence="short",
                                                 sector=object_info.sectors).download_all()
            elif mission == "TESS":
                tpfs = lk.search_targetpixelfile(str(mission_id), mission=mission, cadence="short").download_all()
            elif object_info.sectors != 'all':
                tpfs = lk.search_targetpixelfile(str(mission_id), mission=mission, cadence="short",
                                                 quarter=object_info.sectors).download_all()
            else:
                tpfs = lk.search_targetpixelfile(str(mission_id), mission=mission, cadence="short").download_all()
            apertures = {}
            if isinstance(object_info.aperture_file, str):
                aperture = []
                with open(object_info.aperture_file, 'r') as fd:
                    reader = csv.reader(fd)
                    for row in reader:
                        aperture.append(row)
                    aperture = np.array(aperture)
                for tpf in tpfs:
                    if mission_prefix == self.MISSION_ID_KEPLER:
                        apertures[tpf.quarter] = aperture
                    elif mission_prefix == self.MISSION_ID_TESS:
                        apertures[tpf.sector] = aperture
                    elif mission_prefix == self.MISSION_ID_KEPLER_2:
                        apertures[tpf.campaign] = aperture
            else:
                for sector, aperture_file in object_info.aperture_file.items():
                    aperture = []
                    with open(aperture_file, 'r') as fd:
                        reader = csv.reader(fd)
                        for row in reader:
                            aperture.append(row)
                    aperture = np.array(aperture)
                    apertures[sector] = aperture
            lc = None
            for tpf in tpfs:
                if mission_prefix == self.MISSION_ID_KEPLER:
                    sector = tpf.quarter
                elif mission_prefix == self.MISSION_ID_TESS:
                    sector = tpf.sector
                elif mission_prefix == self.MISSION_ID_KEPLER_2:
                    sector = tpf.campaign
                aperture = apertures[sector].astype(bool)
                tpf.plot(aperture_mask=aperture, mask_color='red')
                plt.savefig(sherlock_dir + "/Aperture_[" + str(sector) + "].png")
                plt.close()
                if mission_prefix == self.MISSION_ID_KEPLER:
                    corrector = lk.KeplerCBVCorrector(tpf)
                    corrector.plot_cbvs([1, 2, 3, 4, 5, 6, 7])
                    raw_lc = tpf.to_lightcurve(aperture_mask=aperture).remove_nans()
                    plt.savefig(sherlock_dir + "/Corrector_components[" + str(sector) + "].png")
                    plt.close()
                    it_lc = corrector.correct([1, 2, 3, 4, 5])
                    ax = raw_lc.plot(color='C3', label='SAP Flux', linestyle='-')
                    it_lc.plot(ax=ax, color='C2', label='CBV Corrected SAP Flux', linestyle='-')
                    plt.savefig(sherlock_dir + "/Raw_vs_CBVcorrected_lc[" + str(sector) + "].png")
                    plt.close()
                elif mission_prefix == self.MISSION_ID_KEPLER_2:
                    raw_lc = tpf.to_lightcurve(aperture_mask=aperture).remove_nans()
                    it_lc = raw_lc.to_corrector("sff").correct(windows=20)
                    ax = raw_lc.plot(color='C3', label='SAP Flux', linestyle='-')
                    it_lc.plot(ax=ax, color='C2', label='CBV Corrected SAP Flux', linestyle='-')
                    plt.savefig(sherlock_dir + "/Raw_vs_SFFcorrected_lc[" + str(sector) + "].png")
                    plt.close()
                elif mission_prefix == self.MISSION_ID_TESS:
                    temp_lc = tpf.to_lightcurve(aperture_mask=aperture)
                    where_are_NaNs = np.isnan(temp_lc.flux)
                    temp_lc = temp_lc[np.where(~where_are_NaNs)]
                    regressors = tpf.flux[np.argwhere(~where_are_NaNs), ~aperture]
                    temp_token_lc = [temp_lc[i: i + 2000] for i in range(0, len(temp_lc), 2000)]
                    regressors_token = [regressors[i: i + 2000] for i in range(0, len(regressors), 2000)]
                    it_lc = None
                    raw_it_lc = None
                    item_index = 0
                    for temp_token_lc_item in temp_token_lc:
                        regressors_token_item = regressors_token[item_index]
                        design_matrix = lk.DesignMatrix(regressors_token_item, name='regressors').pca(5).append_constant()
                        corr_lc = lk.RegressionCorrector(temp_token_lc_item).correct(design_matrix)
                        if it_lc is None:
                            it_lc = corr_lc
                            raw_it_lc = temp_token_lc_item
                        else:
                            it_lc = it_lc.append(corr_lc)
                            raw_it_lc = raw_it_lc.append(temp_token_lc_item)
                        item_index = item_index + 1
                    ax = raw_it_lc.plot(label='Raw light curve')
                    it_lc.plot(ax=ax, label='Corrected light curve')
                    plt.savefig(sherlock_dir + "/Raw_vs_DMcorrected_lc[" + str(sector) + "].png")
                    plt.close()
                if lc is None:
                    lc = it_lc.normalize()
                else:
                    lc = lc.append(it_lc.normalize())
            lc = lc.remove_nans()
            lc.plot(label="Normalized light curve")
            plt.savefig(sherlock_dir + "/Normalized_lc[" + str(sector) + "].png")
            plt.close()
            transits_min_count = 1 if len(tpfs) == 0 else 2
            if mission_prefix == self.MISSION_ID_KEPLER or mission_id == self.MISSION_ID_KEPLER_2:
                quarters = [lcfile.quarter for lcfile in tpfs]
            elif mission_prefix == self.MISSION_ID_TESS:
                sectors = [file.sector for file in tpfs]
            if mission_prefix == self.MISSION_ID_KEPLER_2:
                logging.info("Correcting K2 motion in light curve...")
                quarters = [lcfile.campaign for lcfile in tpfs]
            return lc, star_info, transits_min_count, np.unique(sectors), np.unique(quarters)




'''

This script is designed to test that the data is downloaded correctly.


run with: python -m unittest tests/test_data_download.py

# NOTE: requires intenet connection

'''

import os
import unittest
import requests
import warnings

warnings.filterwarnings("ignore")
from LATTE import LATTEutils

# get the outir (or indir) path
syspath = str(os.path.abspath(LATTEutils.__file__))[0:-14]

with open("{}/_config.txt".format(syspath), 'r') as f:
	outdir = str(f.readlines()[-1])

class TestTESSpoint(unittest.TestCase):

	'''
	Test that TESS point returns the right RA and Dec

	Note: requires internet connection
	'''
	def test_testpoint(self):
		output = LATTEutils.tess_point(outdir, '55525572')

		self.assertAlmostEqual(output[1], 72.69404300000001,places=5,  msg ="TESS point RA is not correct")
		self.assertAlmostEqual(output[2], -60.905449, places=4, msg ="TESS point Dec is not correct")


class TestNearestNeighbours(unittest.TestCase):

	'''
	test that the code can calculate the nearest neighbour information correctly - involves calculations so ensures that they are correct and work as expected.
	'''
	def test_nn(self):
		output = LATTEutils.nn_ticids(outdir, [5], '55525572')

		self.assertEqual(output[0], [55525572, 55525518, 358806415, 55557836, 55557135, 55522986], msg="nearest neighbour TIC IDs are incorrect")

		self.assertEqual(output[1][0], 0.0, msg="nearest neighbour distances are incorrect")
		self.assertAlmostEqual(output[1][1], 6.795727328666717, places=5, msg="nearest neighbour distances are incorrect")
						
		self.assertAlmostEqual(output[1][2], 10.622509290130298,places=5, msg="nearest neighbour distances are incorrect")
		self.assertAlmostEqual(output[1][3], 15.63959237521102, places=5, msg="nearest neighbour distances are incorrect")
						

		self.assertAlmostEqual(output[2], 72.6941, places=3, msg="RA is incorrect")
		self.assertAlmostEqual(output[3], -60.9055, places=3, msg="DEC is incorrect")


class TestDownloadLC(unittest.TestCase):
	'''
	test to ensure that the data download works as it should 
	- test that it return the right file withthe right values.

	NOTE: will only work if the internet connection works. 

	'''
	def test_downloadLC(self):
		alltime, allflux, allflux_err, all_md, alltimebinned, allfluxbinned, allx1, allx2, ally1, ally2, alltimel2, allfbkg, start_sec, end_sec, in_sec, tessmag, teff, srad = LATTEutils.download_data(outdir, [5], '55525572', binfac = 5)


		self.assertAlmostEqual(float(alltime[0]), 1437.9785214992496, places=5, msg="alltime is incorrect")
		self.assertAlmostEqual(float(allflux[1000]), 1.0011740922927856, places=5, msg="allflux is incorrect")
		self.assertAlmostEqual(float(allflux_err[1000]), 0.0009966295910999179, places=5,msg= "allflux_err is incorrect")
		self.assertAlmostEqual(float(all_md[0]), 1441.0243360822994, places=5, msg="all_md is incorrect")
		self.assertAlmostEqual(float(alltimebinned[0]), 1437.9812992567897, places=5, msg="alltimebinned is incorrect")
		self.assertAlmostEqual(float(allfluxbinned[1000]), 1.000087034702301, places=5, msg="alltimebinned is incorrect")
		
		self.assertAlmostEqual(float(allx1[0]), -0.006673532877357502,places=5,  msg="allx1 is incorrect")
		self.assertAlmostEqual(float(allx2[0]), -0.008006655611097813, places=5, msg="allx2 is incorrect")
		self.assertAlmostEqual(float(ally1[0]), -0.018879381330179967, places=5,msg= "ally1 is incorrect")
		self.assertAlmostEqual(float(ally2[0]), -0.02256738394498825, places=5, msg="ally2 is incorrect")
		self.assertAlmostEqual(float(alltimel2[0]), 1437.9924102871835, places=5, msg="alltimel2 is incorrect")
		self.assertAlmostEqual(float(allfbkg[1000]), 974.2296752929688, places=5, msg="allfbkg is incorrect")
		
		self.assertAlmostEqual(float(start_sec[0][0]), 1437.9785214992496, places=5, msg="start_sec is incorrect")
		self.assertAlmostEqual(float(end_sec[0][0]), 1464.2879709506096, places=5, msg="end_sec is incorrect")
		self.assertAlmostEqual(float(in_sec[0]), 5.0, msg="in_sec is incorrect")

		self.assertAlmostEqual(float(tessmag), 9.81999969, places=5, msg="tessmag is incorrect")
		self.assertAlmostEqual(float(teff), 5823.70019531, places=5, msg="teff is incorrect")
		self.assertAlmostEqual(float(srad), 1.93253005, places=5, msg="srad is incorrect")


if __name__ == '__main__':

	unittest.main()




import pytest
import warnings
import numpy as np

from ..source import Source
from ..targetdata import TargetData
from ..visualize import Visualize

def test_making_tpfs():
    '''Does providing an RA/Dec pair, Gaia ID, or TIC ID for the
    SAME target produce the SAME light curve?"
    '''

    star1 = Source(coords=[63.37389, -69.226789], sector=1, tc=True)
    star2 = Source(gaia=4666498154837086208, sector=1, tc=True)
    star3 = Source(tic=25155310, sector=1, tc=True)

    data1 = TargetData(star1)
    data2 = TargetData(star2)
    data3 = TargetData(star3)

    test1 = np.sum(data1.raw_flux - data2.raw_flux) # should be exactly zero
    test2 = np.sum(data2.raw_flux - data3.raw_flux) # should be zero

    assert(test1 == test2)
    assert(test2 == 0)

def test_arb_size_tpfs():
    star = Source(tic=29987116, sector=1, tc=True)
    data = TargetData(star, height=15, width=13)
    assert(np.shape(data.raw_flux[0] == (15,13))) # eleanor enforces oddness





# -----------------------------
'''
A test to see whether the DV reports are generated as we would expect them to be. 
This uses previously generated input files and therfore doesn't test the generatino of the files, but only the compilation of the report. 
'''

import os
import sys
import time 
import datetime
import unittest

from dateutil import parser

import warnings
warnings.filterwarnings("ignore")

#custom modules to import
from LATTE import LATTE_DV as ldv

# test the downloading of the data
# get the outir (or indir) path
syspath = str(os.path.abspath(ldv.__file__))[0:-14]

indir = "./LATTE/test_output"		

# -------------
# test with these input parameters (this data is already downloaded so it's only testing that the plotting part works)
tic = '55525572'
sector = '5'
sectors = [5]
sectors_all = [1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13]
transit_list = [1454.7]
target_ra = 72.6941
target_dec = -60.9055

tessmag = 9.8200
teff  = 5824
srad  = 1.9325
tpf_corrupt  = False
astroquery_corrupt = False
# -------------

# create a mock 'argparser' becasuse the argparse function doesn't work within the unittest module 
# for now use all of the default values. 
class Namespace:
	def __init__(self, **kwargs):  # arbitrary number of input values
		self.__dict__.update(kwargs)

global args
args = Namespace(new_data=False, tic='no',sector='no', targetlist='no', 
	noshow=True, o=False, auto=False,nickname='noname', FFI=False, save=True,north=False,new_path=False,mpi=False)

# test it with the default arguments (except noshow - we don't want the plots to show up in the test run)
# --------------

class TestDVreport(unittest.TestCase):
	
	def test_DVreport(self):

		ldv.LATTE_DV(tic, indir, syspath, transit_list, sectors_all, target_ra, target_dec, tessmag, teff, srad, [0], [0], tpf_corrupt, astroquery_corrupt, FFI = False,  bls = False, model = False, mpi = None, test = './LATTE/tests/')

		# now that it has been run, test to make sure that the report was created. 
		# check that a NEW report was made, and not just an old one that exists from a previous test run 
		DV_path = '{}/55525572/DV_report_55525572.pdf'.format(indir)

		# Get file's Last modification time stamp only in terms of seconds since epoch 
		time_created_DV = os.path.getmtime(DV_path)

		# Convert seconds since epoch to readable timestamp
		t_create_DV = parser.parse(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time_created_DV)))
		
		# the time now (needed to get the time since creation)
		t_now = (datetime.datetime.now())

		# -------
		# time difference in minutes
		time_since_creation_DV =  ((t_now - t_create_DV).seconds / 60)

		# make sure that the new DV report was made less than 0.1 minutes ago - we want to make sure it's a new file and not am old one.
		self.assertLess(time_since_creation_DV, 0.1, "No full LC plot was generated in the last 60 seconds") # a less than b


if __name__ == '__main__':

	unittest.main()






class StarInfo:
    def __init__(self, object_id=None, ld_coefficients=None, teff=None, lum=None, logg=None, radius=None, radius_min=None,
                 radius_max=None, mass=None, mass_min=None, mass_max=None, ra=None, dec=None):
        self.object_id = object_id
        self.ld_coefficients = ld_coefficients
        self.teff = teff
        self.lum = lum
        self.logg = logg
        self.mass = mass
        self.radius = radius
        self.mass = mass
        self.mass_min = None if mass is None or mass_min is None else mass - mass_min
        self.mass_max = None if mass is None or mass_max is None else mass + mass_max
        self.radius = radius
        self.radius_min = None if radius is None or radius_min is None else radius - radius_min
        self.radius_max = None if radius is None or radius_max is None else radius + radius_max
        self.mass_assumed = False
        self.radius_assumed = False
        self.ra = ra
        self.dec = dec

    def assume_model_mass(self, mass=0.1):
        self.mass = mass
        self.mass_min = mass
        self.mass_max = mass
        self.mass_assumed = True

    def assume_model_radius(self, radius=0.1):
        self.radius = radius
        self.radius_min = radius
        self.radius_max = radius
        self.radius_assumed = True



import logging
import sys
import sherlockpipe.eleanor
sys.modules['eleanor'] = sys.modules['sherlockpipe.eleanor']
import eleanor
from sherlockpipe.eleanor.targetdata import TargetData
import re
import numpy as np
from astropy.coordinates import SkyCoord
from sherlockpipe.star import starinfo
from sherlockpipe.objectinfo.MissionFfiCoordsObjectInfo import MissionFfiCoordsObjectInfo
from sherlockpipe.objectinfo.preparer.LightcurveBuilder import LightcurveBuilder
from sherlockpipe.star.TicStarCatalog import TicStarCatalog
from astropy import units as u
import lightkurve as lk


class MissionFfiLightcurveBuilder(LightcurveBuilder):
    def __init__(self):
        super().__init__()
        self.star_catalog = TicStarCatalog()

    def build(self, object_info, sherlock_dir):
        mission_id = object_info.mission_id()
        sherlock_id = object_info.sherlock_id()
        quarters = None
        sectors = None
        logging.info("Retrieving star catalog info...")
        mission, mission_prefix, id = super().parse_object_id(mission_id)
        transits_min_count = 1
        star_info = None
        quarters = None
        if mission_prefix not in self.star_catalogs:
            raise ValueError("Wrong object id " + mission_id)
        if mission_prefix == self.MISSION_ID_KEPLER or mission_prefix == self.MISSION_ID_KEPLER_2:
            if object_info.sectors != 'all':
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="long",
                                               quarter=object_info.sectors).download_all()
            else:
                lcf = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence="long").download_all()
            lc = lcf.PDCSAP_FLUX.stitch().remove_nans()
            transits_min_count = 1 if len(lcf) == 0 else 2
            if mission_prefix == self.MISSION_ID_KEPLER:
                quarters = [lcfile.quarter for lcfile in lcf]
            elif mission_prefix == self.MISSION_ID_KEPLER_2:
                logging.info("Correcting K2 motion in light curve...")
                quarters = [lcfile.campaign for lcfile in lcf]
                lc = lc.to_corrector("sff").correct(windows=20)
            star_info = starinfo.StarInfo(sherlock_id, *self.star_catalogs[mission_prefix].catalog_info(id))
        else:
            if isinstance(object_info, MissionFfiCoordsObjectInfo):
                coords = SkyCoord(ra=object_info.ra, dec=object_info.dec, unit=(u.deg, u.deg))
                star = eleanor.source.multi_sectors(coords=coords, sectors=object_info.sectors)
            else:
                object_id_parsed = re.search(super().NUMBERS_REGEX, object_info.id)
                object_id_parsed = object_info.id[object_id_parsed.regs[0][0]:object_id_parsed.regs[0][1]]
                star = eleanor.multi_sectors(tic=object_id_parsed, sectors=object_info.sectors)
            if star[0].tic:
                # TODO FIX star info objectid
                logging.info("Assotiated TIC is " + star[0].tic)
                star_info = starinfo.StarInfo(object_info.sherlock_id(), *self.star_catalog.catalog_info(int(star[0].tic)))
            data = []
            for s in star:
                datum = TargetData(s, height=15, width=15, bkg_size=31, do_pca=True)
                data.append(datum)
            quality_bitmask = np.bitwise_and(data[0].quality.astype(int), 175)
            lc = data[0].to_lightkurve(data[0].pca_flux, quality_mask=quality_bitmask).remove_nans().flatten()
            sectors = [datum.source_info.sector for datum in data]
            if len(data) > 1:
                for datum in data[1:]:
                    quality_bitmask = np.bitwise_and(datum.quality, 175)
                    lc = lc.append(datum.to_lightkurve(datum.pca_flux, quality_mask=quality_bitmask).remove_nans().flatten())
                transits_min_count = 2
        return lc, star_info, transits_min_count, sectors, quarters



import sys
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1 import make_axes_locatable
import numpy as np
import warnings, os, requests
import lightkurve as lk
from bs4 import BeautifulSoup
from pylab import *
from astropy.timeseries import LombScargle
from astropy.wcs import WCS
import astropy.units as u
from astropy.coordinates import SkyCoord, Angle

from .ffi import use_pointing_model, load_pointing_model
from .mast import *

__all__ = []


class Visualize(object):
    """
    The main class for creating figures, movies, and interactive plots.
    Allows the user to have a grand ole time playing with their data!

    Parameters
    ----------
    obj :
        Object must have minimum attributes of 2D array of flux.
        Will allow for plotting of both postcards & tpfs.
    obj_type :
        Object type can be set to "tpf" or "postcard". Default is "tpf".
    """

    def __init__(self, object, obj_type="tpf"):
        self.obj      = object
        self.obj_type = obj_type.lower()

        if self.obj_type == "tpf":
            self.flux   = self.obj.tpf
            self.center = (np.nanmedian(self.obj.centroid_xs),
                             np.nanmedian(self.obj.centroid_ys))
            self.dimensions = self.obj.tpf[0].shape
        else:
            self.flux   = self.obj.flux
            self.center = self.obj.center_xy
            self.dimensions = self.obj.dimensions


    def aperture_contour(self, aperture=None, ap_color='w', ap_linewidth=4, ax=None, **kwargs):
        """
        Overplots the countour of an aperture on a target pixel file.
        Contribution from Gijs Mulders.

        Parameters
        ----------
        aperture : np.2darray, optional
            A 2D mask the same size as the target pixel file. Default
            is the eleanor default aperture.
        ap_color : str, optional
            The color of the aperture contour. Takes a matplotlib color.
            Default is red.
        ap_linewidth : int, optional
            The linewidth of the aperture contour. Default is 4.
        ax : matplotlib.axes._subplots.AxesSubplot, optional
            Axes to plot on. 
        """
        if ax == None:
            fig, ax = plt.subplots(nrows=1)

        if aperture is None:
            aperture = self.obj.aperture

        ax.imshow(self.obj.tpf[0], **kwargs)

        f = lambda x,y: aperture[int(y),int(x) ]
        g = np.vectorize(f)

        x = np.linspace(0,aperture.shape[1], aperture.shape[1]*100)
        y = np.linspace(0,aperture.shape[0], aperture.shape[0]*100)
        X, Y= np.meshgrid(x[:-1],y[:-1])
        Z = g(X[:-1],Y[:-1])

        ax.contour(Z, [0.05], colors=ap_color, linewidths=[ap_linewidth],
                    extent=[0-0.5, x[:-1].max()-0.5,0-0.5, y[:-1].max()-0.5])

        if ax == None:
            return fig



    def pixel_by_pixel(self, colrange=None, rowrange=None, cmap='viridis',
                       data_type="corrected", mask=None, xlim=None,
                       ylim=None, color_by_pixel=False, color_by_aperture=True,
                       freq_range=[1/20., 1/0.1], aperture=None, ap_color='r',
                       ap_linewidth=2):
        """
        Creates a pixel-by-pixel light curve using the corrected flux.
        Contribution from Oliver Hall.

        Parameters
        ----------
        colrange : np.array, optional
             A list of start column and end column you're interested in
             zooming in on.
        rowrange : np.array, optional
             A list of start row and end row you're interested in zooming
             in on.
        cmap : str, optional
             Name of a matplotlib colormap. Default is 'viridis'.
        data_type : str, optional
             The type of flux used. Either: 'raw', 'corrected', 'amplitude',
             or 'periodogram'. If not, default set to 'corrected'.
        mask : np.array, optional
             Specifies the cadences used in the light curve. If not, default
             set to good quality cadences.
        xlim : np.array, optional
             Specifies the xlim on the subplots. If not, default is set to
             the entire light curve.
        ylim : np.array, optional
             Specifies the ylim on the subplots, If not, default is set to
             the entire light curve flux range.
        color_by_pixel : bool, optional
             Colors the light curve given the color of the pixel. If not,
             default is set to False.
        freq_range : list, optional
             List of minimum and maximum frequency to search in Lomb Scargle
             periodogram. Only used if data_type = 'periodogram'. If None,
             default = [1/20., 1/0.1].
        """
        if self.obj.lite:
            print('This is an eleanor-lite object. No pixel_by_pixel visualization can be created.')
            print('Please create a regular eleanor.TargetData object (lite=False) to use this tool.')
            return

        if colrange is None:
            colrange = [0, self.dimensions[1]]

        if rowrange is None:
            rowrange = [0, self.dimensions[0]]

        nrows = int(np.round(colrange[1]-colrange[0]))
        ncols = int(np.round(rowrange[1]-rowrange[0]))

        if (colrange[1] > self.dimensions[1]) or (rowrange[1] > self.dimensions[0]):
            raise ValueError("Asking for more pixels than available in the TPF.")


        figure = plt.figure(figsize=(20,8))
        outer = gridspec.GridSpec(1,2, width_ratios=[1,4])

        inner = gridspec.GridSpecFromSubplotSpec(ncols, nrows, hspace=0.1, wspace=0.1,
                                                 subplot_spec=outer[1])

        i, j = rowrange[0], colrange[0]

        if mask is None:
            q = self.obj.quality == 0
        else:
            q = mask == 0


        ## PLOTS TARGET PIXEL FILE ##
            
        ax = plt.subplot(outer[0])
        
        if aperture is None:
            aperture = self.obj.aperture

        plotflux = np.nanmedian(self.flux[:, rowrange[0]:rowrange[1], 
                                          colrange[0]:colrange[1]], axis=0)
        c = ax.imshow(plotflux,
                      vmax=np.percentile(plotflux, 95),
                      cmap=cmap)
        divider = make_axes_locatable(ax)
        cax = divider.append_axes('right', size='5%', pad=0.15)
        plt.colorbar(c, cax=cax, orientation='vertical')
            
        f = lambda x,y: aperture[int(y),int(x) ]
        g = np.vectorize(f)

        x = np.linspace(colrange[0],colrange[1], nrows*100)
        y = np.linspace(rowrange[0],rowrange[1], ncols*100)
        X, Y= np.meshgrid(x[:-1],y[:-1])
        Z = g(X[:-1],Y[:-1])

        ax.contour(Z, [0.05], 
                   colors=ap_color, linewidths=[ap_linewidth],
                    extent=[0-0.5, nrows-0.5,0-0.5, ncols-0.5])

        ## PLOTS PIXEL LIGHT CURVES ##
        for ind in range( int(nrows * ncols) ):
            if ind == 0:                
                ax = plt.Subplot(figure, inner[ind])
                origax = ax
            else:
                ax = plt.Subplot(figure, inner[ind], sharex=origax)

            flux = self.flux[:,i,j]
            time = self.obj.time
            corr_flux = self.obj.corrected_flux(flux=flux)

            if data_type.lower() == 'corrected':
                y = corr_flux[q]/np.nanmedian(corr_flux[q])
                x = time[q]

            elif data_type.lower() == 'amplitude':
                lc = lk.LightCurve(time=time, flux=corr_flux)
                pg = lc.normalize().to_periodogram()
                x = pg.frequency.value
                y = pg.power.value

            elif data_type.lower() == 'raw':
                y = flux[q]/np.nanmedian(flux[q])
                x = time[q]

            elif data_type.lower() == 'periodogram':
                freq, power = LombScargle(time, corr_flux).autopower(minimum_frequency=freq_range[0],
                                                                     maximum_frequency=freq_range[1],
                                                                     method='fast')
                y = power
                x = 1/freq

            if color_by_pixel is False:
                color = 'k'
            else:
                rgb = c.cmap(c.norm(self.flux[100,i,j]))
                color = matplotlib.colors.rgb2hex(rgb)

            ax.plot(x, y, c=color)
            
            if color_by_aperture and aperture[i,j] > 0:
                for iax in ['top', 'bottom', 'left', 'right']:
                    ax.spines[iax].set_color(ap_color)
                    ax.spines[iax].set_linewidth(ap_linewidth)

            j += 1
            if j == colrange[1]:
                i += 1
                j  = colrange[0]

            if ylim is None:
                ax.set_ylim(np.percentile(y, 1), np.percentile(y, 99))
            else:
                ax.set_ylim(ylim[0], ylim[1])

            if xlim is None:
                ax.set_xlim(np.min(x)-0.1, np.max(x)+0.1)
            else:
                ax.set_xlim(xlim[0], xlim[1])

            if data_type.lower() == 'amplitude':
                ax.set_yscale('log')
                ax.set_xscale('log')
                ax.set_ylim(y.min(), y.max())
                ax.set_xlim(np.min(x),
                            np.max(x))

            ax.set_xticks([])
            ax.set_yticks([])

            figure.add_subplot(ax)

        return figure


    def tess_the_movie(self):
        """
        Opens the link to Ethan Kruse's TESS: The Movie YouTube videos for
        the sector your target is observed in.

        Parameters
        ----------

        Attributes
        ----------
        movie_url : str

        """
        def type_of_script():
            try:
                ipy_str = str(type(get_ipython()))
                if 'zmqshell' in ipy_str:
                    return 'jupyter'
                if 'terminal' in ipy_str:
                    return 'ipython'
            except:
                return 'terminal'

        sector = self.obj.source_info.sector

        base="https://www.youtube.com/results?search_query="
        query="TESS+the+movie+sector+{0}+ethankruse".format(sector)

        soup = BeautifulSoup(requests.get(base+query).text, "html.parser").find_all('script')[26]

        items = soup.text
        items = items.split('\n')[1].split('title')

        good_sector=0

        for subitem in items:
            j = subitem.find('Sector')
            if j > 0 and 'TESS: The Movie' in subitem:
                
                sect = subitem[j:j+100].split(',')[0].split(' ')[-1]
                
                if int(sect) == int(sector):
                    i = subitem.find('/watch?v')
                    ext = subitem[i:i+100].split('"')[0]
                    good_sector=1
                    break

        if good_sector == 1:
            self.movie_url = 'https://www.youtube.com{0}'.format(ext)
            
            call_location = type_of_script()
            
            if (call_location == 'terminal') or (call_location == 'ipython'):
                os.system('python -m webbrowser -t "{0}"'.format(self.movie_url))

            elif (call_location == 'jupyter'):
                from IPython.display import YouTubeVideo
                id = self.movie_url.split('=')[-1]
                return YouTubeVideo(id=id, width=900, height=500)

        else:
            print('No movie is available yet.')
            return


    def plot_gaia_overlay(self, tic=None, tpf=None, magnitude_limit=18):
        """Check if the source is contaminated."""

        if tic is None:
            tic = self.obj.source_info.tic

        if tpf is None:
            tpf = lk.search_tesscut(f'TIC {tic}')[0].download(cutout_size=(self.obj.tpf.shape[1],
                                                                           self.obj.tpf.shape[2]))
            
        fig = tpf.plot(show_colorbar=False, title='TIC {0}'.format(tic))
        fig = self._add_gaia_figure_elements(tpf, fig, magnitude_limit=magnitude_limit)

        return fig

    def _add_gaia_figure_elements(self, tpf, fig, magnitude_limit=18):
        """Make the Gaia Figure Elements"""
        # Get the positions of the Gaia sources
        c1 = SkyCoord(tpf.ra, tpf.dec, frame='icrs', unit='deg')
        # Use pixel scale for query size
        pix_scale = 21.0
        # We are querying with a diameter as the radius, overfilling by 2x.
        from astroquery.vizier import Vizier
        Vizier.ROW_LIMIT = -1
        result = Vizier.query_region(c1, catalog=["I/345/gaia2"],
                                     radius=Angle(np.max(tpf.shape[1:]) * pix_scale, "arcsec"))
        no_targets_found_message = ValueError('Either no sources were found in the query region '
                                              'or Vizier is unavailable')
        too_few_found_message = ValueError('No sources found brighter than {:0.1f}'.format(magnitude_limit))
        if result is None:
            raise no_targets_found_message
        elif len(result) == 0:
            raise too_few_found_message
        result = result["I/345/gaia2"].to_pandas()
        result = result[result.Gmag < magnitude_limit]
        if len(result) == 0:
            raise no_targets_found_message
        radecs = np.vstack([result['RA_ICRS'], result['DE_ICRS']]).T
        coords = tpf.wcs.all_world2pix(radecs, 1) ## TODO, is origin supposed to be zero or one?
        year = ((tpf.astropy_time[0].jd - 2457206.375) * u.day).to(u.year)
        pmra = ((np.nan_to_num(np.asarray(result.pmRA)) * u.milliarcsecond/u.year) * year).to(u.arcsec).value
        pmdec = ((np.nan_to_num(np.asarray(result.pmDE)) * u.milliarcsecond/u.year) * year).to(u.arcsec).value
        result.RA_ICRS += pmra
        result.DE_ICRS += pmdec

        # Gently size the points by their Gaia magnitude
        sizes = 10000.0 / 2**(result['Gmag']/2)

        plt.scatter(coords[:, 0]+tpf.column, coords[:, 1]+tpf.row, c='firebrick', alpha=0.5, edgecolors='r', s=sizes)
        plt.scatter(coords[:, 0]+tpf.column, coords[:, 1]+tpf.row, c='None', edgecolors='r', s=sizes)
        plt.xlim([tpf.column, tpf.column+tpf.shape[1]])
        plt.ylim([tpf.row, tpf.row+tpf.shape[2]])

        return fig




import os
from pathlib import Path
import pandas as pd
import requests


class OisManager:
    TOIS_CSV_URL = 'https://exofop.ipac.caltech.edu/tess/download_toi.php?sort=toi&output=csv'
    CTOIS_CSV_URL = 'https://exofop.ipac.caltech.edu/tess/download_ctoi.php?sort=ctoi&output=csv'
    KOIS_LIST_URL = 'https://exofop.ipac.caltech.edu/kepler/targets.php?sort=num-pc&page1=1&ipp1=100000&koi1=&koi2='
    KIC_STAR_URL = 'https://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=keplerstellar&select=kepid,dist'
    KOI_CSV_URL = 'https://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=cumulative'
    EPIC_CSV_URL = 'https://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=k2candidates&' \
                   'select=epic_name,epic_candname,k2c_disp,pl_orbper,st_dist,st_teff,st_logg,st_metfe,st_metratio,' \
                   'st_vsini,st_kep,pl_trandep,pl_trandur,pl_rade,pl_eqt,pl_orbincl,ra_str,dec_str,pl_tranmid'
    TOIS_CSV = 'tois.csv'
    CTOIS_CSV = 'ctois.csv'
    KOIS_CSV = 'kois.csv'
    EPIC_CSV = 'epic_ois.csv'
    KIC_STAR_CSV = 'kic_star.csv'
    ois = None

    def __init__(self):
        home = str(Path.home()) + "/.sherlockpipe/"
        if not os.path.exists(home):
            os.mkdir(home)
        self.tois_csv = home + self.TOIS_CSV
        self.ctois_csv = home + self.CTOIS_CSV
        self.kois_csv = home + self.KOIS_CSV
        self.epic_csv = home + self.EPIC_CSV
        self.kic_star_csv = home + self.KIC_STAR_CSV

    def load_ois(self):
        if not os.path.isfile(self.tois_csv) or not os.path.isfile(self.ctois_csv):
            print("TOIs files are not found. Downloading...")
            self.update_tic_csvs()
            print("TOIs files download is finished!")
        toi_data = pd.read_csv(self.tois_csv)
        ois = toi_data
        ctoi_data = pd.read_csv(self.ctois_csv)
        ois = pd.concat([ois, ctoi_data])
        if not os.path.isfile(self.kois_csv):
            print("KOIs files are not found. Downloading...")
            self.update_kic_csvs()
            print("KOIs files download is finished!")
        koi_data = pd.read_csv(self.kois_csv)
        ois = pd.concat([ois, koi_data])
        if not os.path.isfile(self.epic_csv):
            print("EPIC IDs files are not found. Downloading...")
            self.update_epic_csvs()
            print("EPIC IDs files download is finished!")
        epic_data = pd.read_csv(self.epic_csv)
        ois = pd.concat([ois, epic_data])
        return ois

    def update_tic_csvs(self):
        tic_csv = open(self.tois_csv, 'wb')
        request = requests.get(self.TOIS_CSV_URL)
        tic_csv.write(request.content)
        tic_csv.close()
        tic_csv = open(self.ctois_csv, 'wb')
        request = requests.get(self.CTOIS_CSV_URL)
        tic_csv.write(request.content)
        tic_csv.close()
        toi_data = pd.read_csv(self.tois_csv)
        toi_data['TIC ID'] = 'TIC ' + toi_data['TIC ID'].astype(str)
        toi_data['TOI'] = 'TOI ' + toi_data['TOI'].astype(str)
        toi_data = toi_data.rename(columns={'TOI': 'OI', 'TIC ID': 'Object Id', 'TFOPWG Disposition': 'Disposition'})
        toi_data.to_csv(self.tois_csv, index=False)
        ctoi_data = pd.read_csv(self.ctois_csv)
        ctoi_data['TIC ID'] = 'TIC ' + ctoi_data['TIC ID'].astype(str)
        ctoi_data['CTOI'] = 'CTOI ' + ctoi_data['CTOI'].astype(str)
        ctoi_data = ctoi_data.rename(columns={'CTOI': 'OI', 'TIC ID': 'Object Id', 'TFOPWG Disposition': 'Disposition',
                                 'Duration (hrs)': 'Duration (hours)', 'Duration (hrs) Err': 'Duration (hours) err',
                                 'Period (days) Err': 'Period (days) err', 'Radius (R_Earth)': 'Planet Radius (R_Earth)',
                                 'Depth mmag': 'Depth (mmag)', 'Depth mmag Err': 'Depth (mmag) err',
                                 'Depth ppm': 'Depth (ppm)', 'Depth ppm Err': 'Depth (ppm) err',
                                 'Insolation (Earth Flux)': 'Planet Insolation (Earth Flux)',
                                 'Equilibrium Temp (K)': 'Planet Equil Temp (K)'})
        ctoi_data.to_csv(self.ctois_csv, index=False)
        return self

    def update_kic_csvs(self):
        koi_csv = open(self.kois_csv, 'wb')
        request = requests.get(self.KOI_CSV_URL)
        koi_csv.write(request.content)
        koi_csv.close()
        kic_star_csv = open(self.kic_star_csv, 'wb')
        request = requests.get(self.KIC_STAR_URL)
        kic_star_csv.write(request.content)
        kic_star_csv.close()
        koi_data = pd.read_csv(self.kois_csv)
        koi_data['kepid'] = 'KIC ' + koi_data['kepid'].astype(str)
        koi_data['ra_str'] = koi_data['ra_str'].astype(str).replace("h", ":").replace("m", ":").replace("s", "")
        koi_data['dec_str'] = koi_data['dec_str'].astype(str).replace("d", ":").replace("m", ":").replace("s", "")
        koi_data['koi_time0bk'] = koi_data['koi_time0bk'].astype(float) + 2454833.0
        kic_star_data = pd.read_csv(self.kic_star_csv)
        kic_star_data['dist'] = kic_star_data['dist'].astype(float)
        kic_star_data['kepid'] = 'KIC ' + kic_star_data['kepid'].astype(str)
        kic_star_data = kic_star_data[kic_star_data['dist'].notnull()]
        kic_star_data = kic_star_data.groupby(['kepid'])['dist'].mean()
        koi_data = pd.merge(koi_data, kic_star_data, on='kepid', how='left')
        koi_data = koi_data.rename(columns={'kepoi_name': 'OI', 'kepid': 'Object Id',
                                            'koi_disposition': 'Disposition', 'koi_duration': 'Duration (hours)',
                                            'koi_duration_err1': 'Duration (hours) err',
                                            'koi_period': 'Period (days) err', 'koi_period_err1': 'Period (days) err',
                                            'koi_time0bk': 'Epoch (BJD)', 'koi_time0bk_err1': 'Epoch (BJD) err',
                                            'koi_depth': 'Depth (ppm)', 'koi_depth_err1': 'Depth (ppm) err',
                                            'koi_insol': 'Planet Insolation (Earth Flux)',
                                            'koi_insol_err1': 'Planet Insolation (Earth Flux) err',
                                            'koi_teq': 'Planet Equil Temp (K)',
                                            'koi_teq_err1': 'Planet Equil Temp (K) err',
                                            'ra_str': 'RA', 'dec_str': 'Dec',
                                            'koi_slogg': 'Stellar log(g) (cm/s^2)',
                                            'koi_slogg_err1': 'Stellar log(g) (cm/s^2) err',
                                            'koi_steff': 'Stellar Eff Temp (K)',
                                            'koi_steff_err1': 'Stellar Eff Temp (K) err',
                                            'koi_prad': 'Planet Radius (R_Earth)',
                                            'koi_kepmag': 'Kepler Mag',
                                            'dist': 'Stellar Distance (pc)'})
        koi_data.to_csv(self.kois_csv, index=False)
        return self

    def update_epic_csvs(self):
        epic_csv = open(self.epic_csv, 'wb')
        request = requests.get(self.EPIC_CSV_URL)
        epic_csv.write(request.content)
        epic_csv.close()
        epic_data = pd.read_csv(self.epic_csv)
        epic_data['pl_trandep'] = epic_data['pl_trandep'].astype(float) * 1000000
        epic_data['pl_tranmid'] = epic_data['pl_tranmid'].astype(float) - epic_data['pl_trandur'].astype(float) / 2
        epic_data['ra_str'] = epic_data['ra_str'].astype(str).replace("h", ":").replace("m", ":").replace("s", "")
        epic_data['dec_str'] = epic_data['dec_str'].astype(str).replace("d", ":").replace("m", ":").replace("s", "")
        epic_data = epic_data.rename(columns={'epic_candname': 'OI', 'epic_name': 'Object Id',
                                            'k2c_disp': 'Disposition', 'pl_trandur': 'Duration (hours)',
                                            'pl_orbper': 'Period (days)',
                                            'pl_tranmid': 'Epoch (BJD)',
                                            'pl_trandep': 'Depth (ppm)',
                                            'pl_eqt': 'Planet Equil Temp (K)',
                                            'ra_str': 'RA', 'dec_str': 'Dec',
                                            'st_logg': 'Stellar log(g) (cm/s^2)',
                                            'st_teff': 'Stellar Eff Temp (K)',
                                            'pl_rade': 'Planet Radius (R_Earth)',
                                            'st_dist': 'Stellar Distance (pc)',
                                            'st_metfe': 'Stellar Metallicity'})
        epic_data.to_csv(self.epic_csv, index=False)
        return self



from ..mast import *
from eleanor.mast import (crossmatch_distance, tic_by_contamination,
crossmatch_by_position)

import astropy.units as u
from numpy.testing import assert_almost_equal

pos1 = [19, 50]
pos2 = [20, 49]

def test_crossmatch():
    # is distance between cooordinates calculated correctly?
    assert_almost_equal(crossmatch_distance(pos1, pos2).value, 4292.5, decimal=1)
    # assert(len(crossmatch_by_position(pos1, .01, 'Mast.GaiaDR2.Crossmatch')) == 2)

def test_cone_search():
    # there should be 13 targets in within .01 degrees of pos1
    assert(len(cone_search(pos1, .01, 'Mast.Catalogs.GaiaDR2.Cone')) == 13)

def test_coords():
    # fetch coords from Gaia, make sure they're close
    gaia_id = 403326707889616896
    tic_id = 29987116
    result = coords_from_gaia(gaia_id)
    ra, dec = result[0], result[1]
    assert_almost_equal(ra, 19, decimal=1)
    assert_almost_equal(dec, 50, decimal=1)

    '''
    # now check to make sure these coords point to the correct tic
    assert(int(tic_from_coords((ra, dec))[0]) in [tic_id, 25155310])
    assert(int(gaia_from_coords((ra, dec))) in [gaia_id, 4661584398787754240])
    '''
    # ensure coords have correct dimensions
    assert(len(coords_from_tic(tic_id)[0]) == 2)

def test_contamination():
    # there should only be 1 target with contamination < 5.5 in this region
    assert(len(tic_by_contamination(pos1, 1, [0,1], [0, 7.5])) == 1)
